/*************************************************************************
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 * Copyright 2008 by Nakata, Maho
 *
 * MPACK - multiple precision arithmetic library
 *
 * This file is part of MPACK.
 *
 * MPACK is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 3
 * only, as published by the Free Software Foundation.
 *
 * MPACK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3 for more details
 * (a copy is included in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3 along with MPACK.  If not, see
 * <http://www.gnu.org/licenses/lgpl.html>
 * for a copy of the LGPLv3 License.
 *
 ************************************************************************/
#include <stdio.h>
#include <iostream>
#include <fstream>
#include <cstdio>
#include <cstdlib>
#include <cmath>
#include <cstring>
#include <complex>

#include <mblas.h>
#include "blas.h"
#include "mblas.debug.h"

#define MAX_INCX  10
#define MIN_INCX -10
#define MAX_INCY  10
#define MIN_INCY -10
#define MAX_N     10
#define MAX_ITER  1

void
CRrot_test()
{
    int errorflag = FALSE;
    mpc_class ctemp;
    double dtemp;

    for (int incx = MAX_INCX; incx >= MIN_INCX; incx--) {
	for (int incy = MAX_INCY; incy >= MIN_INCY; incy--) {
	    for (int n = 3; n < MAX_N; n++) {
//              printf("# n:%d incx:%d, incy:%d\n", n, incx, incy);
		mpc_class *cx = new mpc_class[VEC_A(n, incx)];
		mpc_class *cy = new mpc_class[VEC_A(n, incy)];
		complex < double >*cxd =
		    new complex < double >[VEC_A(n, incx)];
		complex < double >*cyd =
		    new complex < double >[VEC_A(n, incy)];

		mpf_class c, s;
		double cd, sd;

		set_random_number(&c, &cd);
		set_random_number(&s, &sd);

		int j = 0;

		while (j < MAX_ITER) {
		    set_random_vector(cx, cxd, VEC_A(n, incx));
		    set_random_vector(cy, cyd, VEC_A(n, incy));
		    CRrot(n, cx, incx, cy, incy, c, s);
		    zdrot_f77(&n, cxd, &incx, cyd, &incy, &cd, &sd);
		    double diff = vec_diff(cx, cxd, VEC_A(n, incx), 1);

		    if (fabs(diff) > EPSILON) {
			printf("error %lf!!\n", diff);
			errorflag = TRUE;
		    }
		    diff = vec_diff(cy, cyd, VEC_A(n, incy), 1);
		    if (fabs(diff) > EPSILON) {
			printf("error %lf!!\n", diff);
			errorflag = TRUE;
		    }
		    j++;
		}
		delete[]cx;
		delete[]cy;
		delete[]cxd;
		delete[]cyd;
	    }
	}
    }
    if (errorflag == TRUE) {
	printf("CRrot test failed...\n");
	exit(-1);
    }
}

int
main(int argc, char *argv[])
{
//initialization
    int default_prec = 200;
    gmp_randstate_t state;

    mpf_set_default_prec(default_prec);
    uniformrandom = new gmp_randclass(gmp_randinit_default);
    VERBOSE = 0;

    CRrot_test();
    printf("CRrot test passed...\n");
}
