/*************************************************************************
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 * Copyright 2008 by Nakata, Maho
 *
 * MPACK - multiple precision arithmetic library
 *
 * This file is part of MPACK.
 *
 * MPACK is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 3
 * only, as published by the Free Software Foundation.
 *
 * MPACK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3 for more details
 * (a copy is included in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3 along with MPACK.  If not, see
 * <http://www.gnu.org/licenses/lgpl.html>
 * for a copy of the LGPLv3 License.
 *
 ************************************************************************/
#include <stdio.h>
#include <iostream>
#include <fstream>
#include <cstdio>
#include <cstdlib>
#include <cmath>
#include <cstring>
#include <complex>

#include <mblas.h>
#include "blas.h"
#include "mblas.debug.h"

#define MIN_INCX -10
#define MAX_INCX  10
#define MAX_N     100
#define MAX_ITER  10

void
CRscal_test()
{

    int errorflag = FALSE;
    mpf_class rtemp;
    double dtemp;

    for (int incx = MIN_INCX; incx <= MAX_INCX; incx++) {
	for (int n = 0; n < MAX_N; n++) {
//          printf("# n:%d incx:%d\n", n, incx);
	    mpc_class *x = new mpc_class[VEC_A(n, incx)];
	    complex < double >*xd = new complex < double >[VEC_A(n, incx)];

	    int j = 0;

	    while (j < MAX_ITER) {
		mpf_class alpha;
		double alphad;

		set_random_number(&alpha, &alphad);
		set_random_vector(x, xd, VEC_A(n, incx));

		CRscal(n, alpha, x, incx);
		zdscal_f77(&n, &alphad, xd, &incx);

		double diff;

		diff = vec_diff(x, xd, VEC_A(n, incx), 1);
		if (fabs(diff) > EPSILON) {
		    printf("error %lf!!\n", diff);
		    errorflag = TRUE;
		}
		j++;
	    }
	    delete[]xd;
	    delete[]x;
	}
    }
    if (errorflag == TRUE) {
	printf("CRscal test failed...\n");
	exit(-1);
    }


}

int
main(int argc, char *argv[])
{
//initialization
    int default_prec = 200;
    gmp_randstate_t state;

    mpf_set_default_prec(default_prec);
    uniformrandom = new gmp_randclass(gmp_randinit_default);
    VERBOSE = 0;

    CRscal_test();
    printf("CRscal test passed...\n");
}
