/*************************************************************************
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 * Copyright 2008 by Nakata, Maho
 *
 * MPACK - multiple precision arithmetic library
 *
 * This file is part of MPACK.
 *
 * MPACK is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 3
 * only, as published by the Free Software Foundation.
 *
 * MPACK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3 for more details
 * (a copy is included in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3 along with MPACK.  If not, see
 * <http://www.gnu.org/licenses/lgpl.html>
 * for a copy of the LGPLv3 License.
 *
 ************************************************************************/

#include <stdio.h>
#include <iostream>
#include <fstream>
#include <cstdio>
#include <cstdlib>
#include <cmath>
#include <cstring>
#include <complex>

#include <mblas.h>
#include "blas.h"
#include "mblas.debug.h"

#define MIN_M -2
#define MAX_M 10
#define MIN_N -2
#define MAX_N 10
#define MIN_KL -2
#define MAX_KL 10
#define MIN_KU -2
#define MAX_KU 10
#define MIN_LDA -2
#define MAX_LDA 10
#define MIN_INCX -3
#define MAX_INCX 3
#define MIN_INCY -3
#define MAX_INCY 3
#define MAX_ITER 9

void
Cgbmv_test3(const char *trans, mpc_class alpha, mpc_class beta)
{
    int errorflag = FALSE;

    for (int incy = MIN_INCY; incy < MAX_INCY; incy++) {
	for (int incx = MIN_INCX; incx <= MAX_INCX; incx++) {
	    for (int lda = MIN_LDA; lda < MAX_LDA; lda++) {
		for (int ku = MIN_KU; ku < MAX_KU; ku++) {
		    for (int kl = MIN_KL; kl < MAX_KL; kl++) {
			for (int n = MIN_N; n < MAX_N; n++) {
			    for (int m = MIN_M; m < MAX_M; m++) {

//                              printf
				("#m is %d, n is %d, kl is %d, ku is %d, lda is %d, incx is %d, incy is %d, trans is %s.\n",
				    m, n, kl, ku, lda, incx, incy, trans);

				mpc_class *x;
				mpc_class *y;
				mpc_class *A;
				complex < double >*xd;
				complex < double >*yd;
				complex < double >*Ad;

				if (Mlsame(trans, "N")) {
				    x = new mpc_class[VEC_A(n, incx)];
				    y = new mpc_class[VEC_A(m, incy)];
				    A = new mpc_class[MAT_A(lda, n)];
				    xd = new complex < double >[VEC_A(n,
					    incx)];
				    yd = new complex < double >[VEC_A(m,
					    incy)];
				    Ad = new complex < double >[MAT_A(lda, n)];
				} else {
				    x = new mpc_class[VEC_A(m, incx)];
				    y = new mpc_class[VEC_A(n, incy)];
				    A = new mpc_class[MAT_A(lda, n)];
				    xd = new complex < double >[VEC_A(m,
					    incx)];
				    yd = new complex < double >[VEC_A(n,
					    incy)];
				    Ad = new complex < double >[MAT_A(lda, n)];
				}

				complex < double >alphad, betad;

				alphad.real() = alpha.re.get_d();
				alphad.imag() = alpha.im.get_d();
				betad.real() = beta.re.get_d();
				betad.imag() = beta.im.get_d();

				for (int iter = 0; iter < MAX_ITER; iter++) {
				    if (Mlsame(trans, "N")) {
					set_random_vector(x, xd, VEC_A(n,
						incx));
					set_random_vector(y, yd, VEC_A(m,
						incy));
					set_random_vector(A, Ad, MAT_A(lda,
						n));
				    } else {
					set_random_vector(x, xd, VEC_A(m,
						incx));
					set_random_vector(y, yd, VEC_A(n,
						incy));
					set_random_vector(A, Ad, MAT_A(lda,
						n));
				    }
				    mpack_errno = 0;
				    Cgbmv(trans, m, n, kl, ku, alpha, A, lda,
					x, incx, beta, y, incy);
				    if (mpack_errno != 0) {
					continue;
				    }
				    zgbmv_f77(trans, &m, &n, &kl, &ku, &alphad,
					Ad, &lda, xd, &incx, &betad, yd,
					&incy);

				    double diff;

				    if (Mlsame(trans, "N")) {
					diff =
					    vec_diff(y, yd, VEC_A(m, incy), 1);
				    } else {
					diff =
					    vec_diff(y, yd, VEC_A(n, incy), 1);
				    }
				    if (fabs(diff) > EPSILON) {
					printf("error %lf!!\n", diff);
					errorflag = TRUE;
				    } else {
					printf("okay\n");
				    }
				}
				delete[]x;
				delete[]y;
				delete[]A;
				delete[]xd;
				delete[]yd;
				delete[]Ad;
			    }
			}
		    }
		}
	    }
	}
    }
    if (errorflag == TRUE) {
	printf("Cgbmv test failed...\n");
	exit(-1);
    } else {
	printf("Cgbmv test passed...\n");
    }

}

void
Cgbmv_test2(const char *trans)
{
    mpc_class alpha;
    mpc_class beta;
    mpc_class Czero, Cone;

    Czero.re = 0.0;
    Czero.im = 0.0;
    Cone.re = 1.0;
    Cone.im = 0.0;

//a=0, b=*;
    alpha = Czero;
    beta = mpc_randomnumber();
    Cgbmv_test3(trans, alpha, beta);

//a=*, b=0;
    alpha = mpc_randomnumber();
    beta = Czero;
    Cgbmv_test3(trans, alpha, beta);

//a=*, b=1;
    alpha = mpc_randomnumber();
    beta = Cone;
    Cgbmv_test3(trans, alpha, beta);

//alpha=*, beta=*
    alpha = mpc_randomnumber();
    beta = mpc_randomnumber();
    Cgbmv_test3(trans, alpha, beta);

//a=0, b=0;
    alpha = Czero;
    beta = Czero;
    Cgbmv_test3(trans, alpha, beta);

//a=0, b=1;
    alpha = Czero;
    beta = Cone;
    Cgbmv_test3(trans, alpha, beta);

//a=1, b=0;
    alpha = Cone;
    beta = Czero;
    Cgbmv_test3(trans, alpha, beta);

//a=1, b=1;
    alpha = Cone;
    beta = Cone;
    Cgbmv_test3(trans, alpha, beta);

//a=1, b=*;
    alpha = Cone;
    beta = mpc_randomnumber();
    Cgbmv_test3(trans, alpha, beta);
}

void
Cgbmv_test()
{
    Cgbmv_test2("C");
    Cgbmv_test2("T");
    Cgbmv_test2("N");
}

int
main(int argc, char *argv[])
{
//initialization
    int default_prec = 200;
    gmp_randstate_t state;

    mpf_set_default_prec(default_prec);
    uniformrandom = new gmp_randclass(gmp_randinit_default);
    VERBOSE = 0;

    Cgbmv_test();
}
