/*************************************************************************
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 * Copyright 2008 by Nakata, Maho
 *
 * MPACK - multiple precision arithmetic library
 *
 * This file is part of MPACK.
 *
 * MPACK is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 3
 * only, as published by the Free Software Foundation.
 *
 * MPACK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3 for more details
 * (a copy is included in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3 along with MPACK.  If not, see
 * <http://www.gnu.org/licenses/lgpl.html>
 * for a copy of the LGPLv3 License.
 *
 ************************************************************************/
#include <stdio.h>
#include <iostream>
#include <fstream>
#include <cstdio>
#include <cstdlib>
#include <cmath>
#include <cstring>
#include <complex>
#include <mblas.h>
#include "blas.h"

#include "mblas.debug.h"

#define MIN_INCX  -3
#define MAX_INCX  3
#define MIN_INCY  -3
#define MAX_INCY  3
#define MIN_N     -1
#define MAX_N     10
#define MIN_M     -1
#define MAX_M     10
#define MIN_LDA   -1
#define MAX_LDA   10
#define MAX_ITER 10

void
Cgeru_test()
{
    int errorflag = FALSE;

    for (int incx = MIN_INCX; incx <= MAX_INCX; incx++) {
	for (int incy = MIN_INCY; incy < MAX_INCY; incy++) {
	    for (int n = MIN_N; n < MAX_N; n++) {
		for (int m = MIN_M; m < MAX_M; m++) {
		    for (int lda = m; lda < MAX_LDA; lda++) {
/*
			printf
			    ("#m is %d, n is %d, lda is %d, incx is %d, incy is %d\n",
			    m, n, lda, incx, incy);
*/
			mpc_class *A;
			mpc_class *x;
			mpc_class *y;
			complex < double >*Ad;
			complex < double >*xd;
			complex < double >*yd;

			A = new mpc_class[MAT_A(lda, n)];
			x = new mpc_class[VEC_A(m, incx)];
			y = new mpc_class[VEC_A(n, incy)];
			Ad = new complex < double >[MAT_A(lda, n)];
			xd = new complex < double >[VEC_A(m, incx)];
			yd = new complex < double >[VEC_A(n, incy)];

			mpc_class alpha;
			complex < double >alphad;

			for (int i = 0; i < MAX_ITER; i++) {
			    set_random_number(&alpha, &alphad);

			    set_random_vector(A, Ad, MAT_A(lda, n));
			    set_random_vector(x, xd, VEC_A(m, incx));
			    set_random_vector(y, yd, VEC_A(n, incy));

			    mpack_errno = 0;
			    Cgeru(m, n, alpha, x, incx, y, incy, A, lda);
			    zgeru_f77(&m, &n, &alphad, xd, &incx, yd, &incy,
				Ad, &lda);
			    if (mpack_errno != 0) {
				break;
			    }

			    double diff;

			    diff = vec_diff(A, Ad, MAT_A(lda, n), 1);
			    if (fabs(diff) > EPSILON) {
				printf("#error %lf!!\n", diff);
				errorflag = TRUE;
			    } else {
//                              printf("#okay\n");
			    }

			}
			delete[]yd;
			delete[]xd;
			delete[]Ad;

			delete[]y;
			delete[]x;
			delete[]A;
		    }
		}
	    }
	}
    }

    if (errorflag == TRUE) {
	printf("Cgeru test failed...\n");
	exit(-1);
    }

}

int
main(int argc, char *argv[])
{
//initialization
    int default_prec = 200;
    gmp_randstate_t state;

    mpf_set_default_prec(default_prec);
    uniformrandom = new gmp_randclass(gmp_randinit_default);

    VERBOSE = 0;		//1 is only possible with incx=incy=1

    Cgeru_test();

    printf("Cgeru test passed...\n");

}
