/*************************************************************************
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 * Copyright 2008 by Nakata, Maho
 *
 * MPACK - multiple precision arithmetic library
 *
 * This file is part of MPACK.
 *
 * MPACK is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 3
 * only, as published by the Free Software Foundation.
 *
 * MPACK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3 for more details
 * (a copy is included in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3 along with MPACK.  If not, see
 * <http://www.gnu.org/licenses/lgpl.html>
 * for a copy of the LGPLv3 License.
 *
 ************************************************************************/
#include <stdio.h>
#include <iostream>
#include <fstream>
#include <cstdio>
#include <cstdlib>
#include <cmath>
#include <cstring>
#include <complex>

#include <mblas.h>
#include "blas.h"
#include "mblas.debug.h"

#define MIN_INCX  -1
#define MAX_INCX   5
#define MIN_INCY  -1
#define MAX_INCY   5
#define MIN_N      2
#define MAX_N      10
#define MAX_ITER 100

void
Chpmv2_test2(const char *uplo)
{
    int errorflag = FALSE;

    for (int incx = MIN_INCX; incx <= MAX_INCX; incx++) {
	for (int incy = MIN_INCY; incy < MAX_INCY; incy++) {
	    for (int n = MIN_N; n < MAX_N; n++) {
//              printf("#n is %d, incx is %d, incy is %d, uplo is %s.\n", n,
//                  incx, incy, uplo);
		mpc_class alpha;
		mpc_class beta;
		mpc_class *x;
		mpc_class *y;
		mpc_class *AP;

		complex < double >alphad;
		complex < double >betad;
		complex < double >*xd;
		complex < double >*yd;
		complex < double >*APd;

		x = new mpc_class[VEC_A(n, incx)];
		y = new mpc_class[VEC_A(n, incy)];
		AP = new mpc_class[VEC_A(n * (n + 1) / 2, 1)];
		xd = new complex < double >[VEC_A(n, incx)];
		yd = new complex < double >[VEC_A(n, incy)];
		APd = new complex < double >[VEC_A(n * (n + 1) / 2, 1)];

		for (int i = 0; i < MAX_ITER; i++) {
		    set_random_vector(AP, APd, VEC_A(n * (n + 1) / 2, 1));
		    set_random_vector(x, xd, VEC_A(n, incx));
		    set_random_vector(y, yd, VEC_A(n, incy));
		    set_random_number(&alpha, &alphad);
		    set_random_number(&beta, &betad);

		    Chpmv(uplo, n, alpha, AP, x, incx, beta, y, incy);
		    zhpmv_f77(uplo, &n, &alphad, APd, xd, &incx, &betad, yd,
			&incy);

		    double diff;

		    diff = vec_diff(AP, APd, VEC_A(n * (n + 1) / 2, 1), 1);
		    if (fabs(diff) > EPSILON) {
			printf("#error %lf!!\n", diff);
			errorflag = TRUE;
		    }
		}

		delete[]APd;
		delete[]yd;
		delete[]xd;
		delete[]AP;
		delete[]y;
		delete[]x;
	    }
	}
    }
    if (errorflag == TRUE) {
	printf("Chpmv test failed...\n");
	exit(-1);
    }
}

void
Chpmv2_test()
{
    Chpmv2_test2("U");
    Chpmv2_test2("L");
}

int
main(int argc, char *argv[])
{
//initialization
    int default_prec = 200;
    gmp_randstate_t state;

    mpf_set_default_prec(default_prec);
    uniformrandom = new gmp_randclass(gmp_randinit_default);
    VERBOSE = 0;

    Chpmv2_test();
    printf("Chpmv2 test passed...\n");
}
