/*************************************************************************
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 * Copyright 2008 by Nakata, Maho
 *
 * MPACK - multiple precision arithmetic library
 *
 * This file is part of MPACK.
 *
 * MPACK is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 3
 * only, as published by the Free Software Foundation.
 *
 * MPACK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3 for more details
 * (a copy is included in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3 along with MPACK.  If not, see
 * <http://www.gnu.org/licenses/lgpl.html>
 * for a copy of the LGPLv3 License.
 *
 ************************************************************************/
#include <stdio.h>
#include <iostream>
#include <fstream>
#include <cstdio>
#include <cstdlib>
#include <cmath>
#include <cstring>
#include <complex>

#include <mblas.h>
#include "blas.h"
#include "mblas.debug.h"

#define MIN_INCX   -5
#define MAX_INCX   10
#define MIN_N      -2
#define MAX_N      50
#define MAX_ITER   10

//#define MIN_INCX   1
//#define MAX_INCX   2
//#define MIN_N      1000
//#define MAX_N      1020
//#define MAX_ITER   10

void
Chpr_test2(const char *uplo)
{
    int errorflag = FALSE;

    for (int incx = MIN_INCX; incx <= MAX_INCX; incx++) {
	for (int n = MIN_N; n <= MAX_N; n++) {
	    mpf_class alpha;
	    mpc_class *x;
	    mpc_class *AP;

	    double alphad;
	    complex < double >*xd;
	    complex < double >*APd;

	    x = new mpc_class[VEC_A(n, incx)];
	    AP = new mpc_class[VEC_A(n * (n + 1) / 2, 1)];
	    xd = new complex < double >[VEC_A(n, incx)];
	    APd = new complex < double >[VEC_A(n * (n + 1) / 2, 1)];

	    for (int i = 0; i < MAX_ITER; i++) {
		set_random_vector(AP, APd, VEC_A(n * (n + 1) / 2, 1));
		set_random_vector(x, xd, VEC_A(n, incx));
		set_random_number(&alpha, &alphad);

		Chpr(uplo, n, alpha, x, incx, AP);
		zhpr_f77(uplo, &n, &alphad, xd, &incx, APd);

		double diff;

		diff = vec_diff(AP, APd, VEC_A(n * (n + 1) / 2, 1), 1);
		if (fabs(diff) > EPSILON2) {
		    printf("#error %.16lf!!\n", diff);
		    printf("#n is %d, incx is %d, uplo is %s.\n", n, incx,
			uplo);
		    errorflag = TRUE;
		}
	    }

	    delete[]APd;
	    delete[]xd;
	    delete[]AP;
	    delete[]x;
	}
    }
    if (errorflag) {
	printf("Chpr test has been failed\n");
	exit(-1);
    }
}

void
Chpr_test()
{
    Chpr_test2("L");
    Chpr_test2("U");
}

int
main(int argc, char *argv[])
{
//initialization
    int default_prec = 200;
    gmp_randstate_t state;

    mpf_set_default_prec(default_prec);
    uniformrandom = new gmp_randclass(gmp_randinit_default);
    VERBOSE = 0;

    Chpr_test();
    printf("Chpr test has been passed\n");
}
