/*************************************************************************
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 * Copyright 2008 by Nakata, Maho
 *
 * MPACK - multiple precision arithmetic library
 *
 * This file is part of MPACK.
 *
 * MPACK is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 3
 * only, as published by the Free Software Foundation.
 *
 * MPACK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3 for more details
 * (a copy is included in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3 along with MPACK.  If not, see
 * <http://www.gnu.org/licenses/lgpl.html>
 * for a copy of the LGPLv3 License.
 *
 ************************************************************************/
#include <stdio.h>
#include <iostream>
#include <fstream>
#include <cstdio>
#include <cstdlib>
#include <cmath>
#include <cstring>
#include <complex>

#include <mblas.h>
#include "blas.h"
#include "mblas.debug.h"

#define MAX_ITER 1000

void
Crotg_test()
{
    int errorflag = FALSE;

    mpc_class ca;
    mpc_class cb;
    mpf_class cc;
    mpc_class cs;

    complex < double >cad;
    complex < double >cbd;
    double ccd;
    complex < double >csd;
    int j = 0;

    while (j < MAX_ITER) {
	set_random_number(&ca, &cad);
	set_random_number(&cb, &cbd);
	set_random_number(&cc, &ccd);

	Crotg(&ca, cb, &cc, &cs);
	zrotg_f77(&cad, &cbd, &ccd, &csd);

	double diff[8];

	diff[0] = ca.re.get_d() - cad.real();
	diff[1] = ca.im.get_d() - cad.imag();

	diff[2] = cb.re.get_d() - cbd.real();
	diff[3] = cb.im.get_d() - cbd.imag();

	diff[4] = cc.get_d() - ccd;
	diff[5] = 0.0;

	diff[6] = cs.re.get_d() - csd.real();
	diff[7] = cs.im.get_d() - csd.imag();
/*
	for (int p = 0; p < 8; p++) {
	    printf("diff[%d]=%lf\n", p, diff[p]);
	}
*/
	for (int p = 0; p < 8; p++) {
	    if (fabs(diff[p]) > EPSILON) {
		printf("error %d, %lf!!\n", p, diff[p]);
		errorflag = TRUE;
	    }
	}
	j++;
    }

    if (errorflag == TRUE) {
	printf("Crotg test failed...\n");
	exit(-1);
    }
}

int
main(int argc, char *argv[])
{
//initialization
    int default_prec = 200;
    gmp_randstate_t state;

    mpf_set_default_prec(default_prec);
    uniformrandom = new gmp_randclass(gmp_randinit_default);
    VERBOSE = 0;

    Crotg_test();
    printf("Crotg test passed...\n");
}
