/*************************************************************************
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 * Copyright 2008 by Nakata, Maho
 *
 * MPACK - multiple precision arithmetic library
 *
 * This file is part of MPACK.
 *
 * MPACK is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 3
 * only, as published by the Free Software Foundation.
 *
 * MPACK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3 for more details
 * (a copy is included in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3 along with MPACK.  If not, see
 * <http://www.gnu.org/licenses/lgpl.html>
 * for a copy of the LGPLv3 License.
 *
 ************************************************************************/
#include <stdio.h>
#include <iostream>
#include <fstream>
#include <cstdio>
#include <cstdlib>
#include <cmath>
#include <cstring>
#include <complex>

#include <mblas.h>
#include "blas.h"
#include "mblas.debug.h"

#define MIN_N -1
#define MAX_N 12
#define MAX_K 12
#define MIN_LDA 1
#define MAX_LDA 12
#define MIN_INCX -1
#define MAX_INCX 3
#define MAX_ITER 3

void
Ctbsv_test2(const char *uplo, const char *trans, const char *diag)
{
    int errorflag = FALSE;

    for (int incx = MIN_INCX; incx <= MAX_INCX; incx++) {
	for (int n = MIN_N; n <= MAX_N; n++) {
	    for (int k = 0; k <= MAX_K; k++) {
		for (int lda = k + 1; lda <= MAX_LDA; lda++) {
		    mpc_class *A = new mpc_class[MAT_A(lda, n)];
		    mpc_class *x = new mpc_class[VEC_A(n, incx)];
		    complex < double >*Ad =
			new complex < double >[MAT_A(lda, n)];
		    complex < double >*xd =
			new complex < double >[VEC_A(n, incx)];

		    for (int iter = 0; iter < MAX_ITER; iter++) {
			set_random_vector(A, Ad, MAT_A(lda, n));
			set_random_vector(x, xd, VEC_A(n, incx));
			mpack_errno = 0;
			Ctbsv(uplo, trans, diag, n, k, A, lda, x, incx);
			if (mpack_errno != 0) {
			    break;
			}
			ztbsv_f77(uplo, trans, diag, &n, &k, Ad, &lda, xd,
			    &incx);

			double diff;

			diff = vec_diff(x, xd, VEC_A(n, incx), 1);
			if (fabs(diff) > EPSILON2) {
			    printf("#error %lf!!\n", diff);
			    printf("#n is %d, lda is %d, incx is %d ", n, lda,
				incx);
			    printf("uplo is %s trans is %s, diag is %s \n",
				uplo, trans, diag);
			    errorflag = TRUE;
			}

		    }
		    delete[]Ad;
		    delete[]xd;
		    delete[]x;
		    delete[]A;
		}
	    }
	}
    }
    if (errorflag == TRUE) {
	printf("Ctbsv test failed...\n");
	exit(-1);
    }
}
void
Ctbsv_test()
{

    Ctbsv_test2("U", "N", "U");
    Ctbsv_test2("U", "N", "N");
    Ctbsv_test2("U", "T", "U");
    Ctbsv_test2("U", "T", "N");
    Ctbsv_test2("U", "C", "U");
    Ctbsv_test2("U", "C", "N");

    Ctbsv_test2("L", "N", "U");
    Ctbsv_test2("L", "N", "N");
    Ctbsv_test2("L", "T", "U");
    Ctbsv_test2("L", "T", "N");
    Ctbsv_test2("L", "C", "U");
    Ctbsv_test2("L", "C", "N");

}


int
main(int argc, char *argv[])
{
//initialization
    int default_prec = 200;
    gmp_randstate_t state;

    mpf_set_default_prec(default_prec);
    uniformrandom = new gmp_randclass(gmp_randinit_default);
    Ctbsv_test();
    printf("Ctbsv test passed...\n");
}
