/*************************************************************************
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 * Copyright 2008 by Nakata, Maho
 *
 * MPACK - multiple precision arithmetic library
 *
 * This file is part of MPACK.
 *
 * MPACK is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 3
 * only, as published by the Free Software Foundation.
 *
 * MPACK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3 for more details
 * (a copy is included in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3 along with MPACK.  If not, see
 * <http://www.gnu.org/licenses/lgpl.html>
 * for a copy of the LGPLv3 License.
 *
 ************************************************************************/
#include <stdio.h>
#include <iostream>
#include <fstream>
#include <cstdio>
#include <cstdlib>
#include <cmath>
#include <cstring>
#include <complex>

#include <mblas.h>
#include "blas.h"
#include "mblas.debug.h"

#define MIN_N -1
#define MAX_N 20
#define MIN_INCX -3
#define MAX_INCX 3
#define MAX_ITER 5

void
Ctpsv_test2(const char *uplo, const char *trans, const char *diag)
{
    int errorflag = FALSE;

    for (int incx = MIN_INCX; incx <= MAX_INCX; incx++) {
	for (int n = MIN_N; n < MAX_N; n++) {
/*	    printf("#n is %d, incx is %d ", n, incx);
	    printf("uplo is %s trans is %s, diag is %s \n", uplo, trans, diag); */

	    mpc_class *AP = new mpc_class[VEC_A(n * (n + 1) / 2, 1)];
	    mpc_class *x = new mpc_class[VEC_A(n, incx)];
	    complex < double >*APd =
		new complex < double >[VEC_A(n * (n + 1) / 2, 1)];
	    complex < double >*xd = new complex < double >[VEC_A(n, incx)];

	    for (int iter = 0; iter < MAX_ITER; iter++) {
		set_random_vector(AP, APd, VEC_A(n * (n + 1) / 2, 1));
		set_random_vector(x, xd, VEC_A(n, incx));

		mpack_errno = 0;
		Ctpsv(uplo, trans, diag, n, AP, x, incx);
		ztpsv_f77(uplo, trans, diag, &n, APd, xd, &incx);
		if (mpack_errno != 0) {
		    break;
		}

		double diff;

		diff = vec_diff(x, xd, VEC_A(n, incx), 1);
		if (fabs(diff) > EPSILON3) {
		    printf("#error %.16lf!!\n", diff);
		    errorflag = TRUE;
		} else {
//                  printf("#okay\n");
		}
	    }
	    delete[]APd;
	    delete[]AP;
	    delete[]xd;
	    delete[]x;

	}
    }
    if (errorflag == TRUE) {
	printf("Ctpsv test failed...\n");
	exit(-1);
    }
}

void
Ctpsv_test()
{
    Ctpsv_test2("U", "N", "U");
    Ctpsv_test2("U", "N", "N");
    Ctpsv_test2("U", "T", "U");
    Ctpsv_test2("U", "T", "N");
    Ctpsv_test2("U", "C", "U");
    Ctpsv_test2("U", "C", "N");

    Ctpsv_test2("L", "N", "U");
    Ctpsv_test2("L", "N", "N");
    Ctpsv_test2("L", "T", "U");
    Ctpsv_test2("L", "T", "N");
    Ctpsv_test2("L", "C", "U");
    Ctpsv_test2("L", "C", "N");

}


int
main(int argc, char *argv[])
{
//initialization
    int default_prec = 200;
    gmp_randstate_t state;

    mpf_set_default_prec(default_prec);
    uniformrandom = new gmp_randclass(gmp_randinit_default);
    Ctpsv_test();
    printf("Ctpsv test passed...\n");

}
