/*************************************************************************
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 * Copyright 2008 by Nakata, Maho
 *
 * MPACK - multiple precision arithmetic library
 *
 * This file is part of MPACK.
 *
 * MPACK is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 3
 * only, as published by the Free Software Foundation.
 *
 * MPACK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3 for more details
 * (a copy is included in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3 along with MPACK.  If not, see
 * <http://www.gnu.org/licenses/lgpl.html>
 * for a copy of the LGPLv3 License.
 *
 ************************************************************************/
#include <stdio.h>
#include <iostream>
#include <fstream>
#include <cstdio>
#include <cstdlib>
#include <cmath>
#include <cstring>
#include <complex>

#include <mblas.h>
#include "blas.h"
#include "mblas.debug.h"

#define MAX_ITER 10

void
RCabs1_test()
{
    int errorflag = FALSE;

    VERBOSE = 0;

    mpc_class z;
    complex < double >dz;

    mpf_class Ftemp;
    double dtemp;

    int j = 0;

    while (j < MAX_ITER) {

	z = mpc_randomnumber();
	dz.real() = z.re.get_d();
	dz.imag() = z.im.get_d();

	Ftemp = RCabs1(z);
	dtemp = dcabs1_f77(&dz);

	if (VERBOSE) {
	    printf("Z  =");
	    gmp_printf(GMP_P_FORMAT, z.re.get_mpf_t());
	    gmp_printf(GMP_P_FORMAT, z.im.get_mpf_t());
	    printf("i\n");

	    printf("dz =");
	    printf(P_FORMAT, dz.real());
	    printf(P_FORMAT, dz.imag());
	    printf("i\n");

	    printf("RCabs1(Z)  is ");
	    gmp_printf(GMP_P_FORMAT, Ftemp.get_mpf_t());
	    printf("\n");
	    printf("dcabs1(dz) is ");
	    printf(P_FORMAT, dtemp);
	    printf("\n");
	    printf("\n\n");
	}
	double diff1 = Ftemp.get_d() - dtemp;

	if (fabs(diff1) > EPSILON) {
	    printf("error 1 !!\n");
	    errorflag = TRUE;
	}
	j++;
    }

    if (errorflag == TRUE) {
	printf("RCabs1 test failed...\n");
    } else {
	printf("RCabs1 test passed...\n");
    }


}

int
main(int argc, char *argv[])
{
//initialization
    int default_prec = 200;
    gmp_randstate_t state;

    mpf_set_default_prec(default_prec);
    uniformrandom = new gmp_randclass(gmp_randinit_default);
    VERBOSE = 0;

    RCabs1_test();
}
