/*************************************************************************
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 * Copyright 2008 by Nakata, Maho
 *
 * MPACK - multiple precision arithmetic library
 *
 * This file is part of MPACK.
 *
 * MPACK is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 3
 * only, as published by the Free Software Foundation.
 *
 * MPACK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3 for more details
 * (a copy is included in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3 along with MPACK.  If not, see
 * <http://www.gnu.org/licenses/lgpl.html>
 * for a copy of the LGPLv3 License.
 *
 ************************************************************************/
#include <stdio.h>
#include <iostream>
#include <fstream>
#include <cstdio>
#include <cstdlib>
#include <cmath>
#include <cstring>
#include <complex>

#include <mblas.h>
#include "blas.h"

#include "mblas.debug.h"

#define MAX_INCX  10
#define MIN_INCX -10
#define MAX_INCY  10
#define MIN_INCY -10
#define MAX_N     10
#define MAX_ITER  10

void
Raxpy_test2(mpf_class alpha)
{

    int errorflag = FALSE;
    mpf_class ctemp;
    double dtemp;

    double alphad;

    alphad = alpha.get_d();

    for (int incx = MAX_INCX; incx >= MIN_INCX; incx--) {
	for (int incy = MAX_INCY; incy >= MIN_INCY; incy--) {
	    for (int n = 0; n < MAX_N; n++) {
//              printf("# n:%d incx:%d, incy:%d\n", n, incx, incy);

		mpf_class *x = new mpf_class[VEC_A(n, incx)];
		mpf_class *y = new mpf_class[VEC_A(n, incy)];
		double *xd = new double[VEC_A(n, incx)];
		double *yd = new double[VEC_A(n, incy)];
		int j = 0;

		while (j < MAX_ITER) {
		    set_random_vector(x, xd, VEC_A(n, incx));
		    set_random_vector(y, yd, VEC_A(n, incy));

		    Raxpy(n, alpha, x, incx, y, incy);
		    daxpy_f77(&n, &alphad, xd, &incx, yd, &incy);

		    double diff = vec_diff(y, yd, VEC_A(n, incy), 1);

		    if (fabs(diff) > EPSILON) {
			printf("error %lf!!\n", diff);
			errorflag = TRUE;
		    }
		    j++;
		}
		delete[]x;
		delete[]xd;
		delete[]y;
		delete[]yd;
	    }
	}
    }
    if (errorflag == TRUE) {
	printf("Raxpy test failed...\n");
	exit(-1);
    }
}

void
Raxpy_test()
{
    mpf_class alpha;

    alpha = 0.0;
    Raxpy_test2(alpha);

    alpha = mpf_randomnumber();
    Raxpy_test2(alpha);
}

int
main(int argc, char *argv[])
{
//initialization
    int default_prec = 200;
    gmp_randstate_t state;

    mpf_set_default_prec(default_prec);
    uniformrandom = new gmp_randclass(gmp_randinit_default);
    VERBOSE = 0;
    Raxpy_test();
    printf("Raxpy test passed...\n");

}
