/*************************************************************************
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 * Copyright 2008 by Nakata, Maho
 *
 * MPACK - multiple precision arithmetic library
 *
 * This file is part of MPACK.
 *
 * MPACK is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 3
 * only, as published by the Free Software Foundation.
 *
 * MPACK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3 for more details
 * (a copy is included in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3 along with MPACK.  If not, see
 * <http://www.gnu.org/licenses/lgpl.html>
 * for a copy of the LGPLv3 License.
 *
 ************************************************************************/
#include <stdio.h>
#include <iostream>
#include <fstream>
#include <cstdio>
#include <cstdlib>
#include <cmath>
#include <cstring>
#include <complex>

#include <mblas.h>
#include "blas.h"
#include "mblas.debug.h"

#define MAX_ITER  100

void
Rrotg_test()
{
    int errorflag = FALSE;

    mpf_class a;
    mpf_class b;
    mpf_class c;
    mpf_class s;

    double ad;
    double bd;
    double cd;
    double sd;
    int j = 0;

    while (j < MAX_ITER) {
	set_random_number(&a, &ad);
	set_random_number(&b, &bd);
	set_random_number(&c, &cd);

	Rrotg(&a, &b, &c, &s);
	drotg_f77(&ad, &bd, &cd, &sd);

	double diff[4];

	diff[0] = a.get_d() - ad;
	diff[1] = b.get_d() - bd;
	diff[2] = c.get_d() - cd;
	diff[3] = s.get_d() - sd;
/*
	for (int p = 0; p < 4; p++) {
	    printf("diff[%d]=%lf\n", p, diff[p]);
	}
*/
	for (int p = 0; p < 4; p++) {
	    if (fabs(diff[p]) > EPSILON) {
		printf("error %d, %lf!!\n", p, diff[p]);
		errorflag = TRUE;
	    }
	}
	j++;
    }
    if (errorflag == TRUE) {
	printf("Rrotg test failed...\n");
	exit(-1);
    }

}

int
main(int argc, char *argv[])
{
//initialization
    int default_prec = 200;
    gmp_randstate_t state;

    mpf_set_default_prec(default_prec);
    uniformrandom = new gmp_randclass(gmp_randinit_default);
    VERBOSE = 0;

    Rrotg_test();
    printf("Rrotg test passed...\n");
}
