/*************************************************************************
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 * Copyright 2008 by Nakata, Maho
 *
 * MPACK - multiple precision arithmetic library
 *
 * This file is part of MPACK.
 *
 * MPACK is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 3
 * only, as published by the Free Software Foundation.
 *
 * MPACK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3 for more details
 * (a copy is included in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3 along with MPACK.  If not, see
 * <http://www.gnu.org/licenses/lgpl.html>
 * for a copy of the LGPLv3 License.
 *
 ************************************************************************/
#include <stdio.h>
#include <iostream>
#include <fstream>
#include <cstdio>
#include <cstdlib>
#include <cmath>
#include <cstring>
#include <complex>

#include <mblas.h>
#include "blas.h"
#include "mblas.debug.h"

#define MAX_INCX  10
#define MIN_INCX -10
#define MAX_INCY  10
#define MIN_INCY -10
#define MAX_N     10
#define MAX_ITER  1

void
Rrotm_test2(mpf_class dflag)
{
    int errorflag = FALSE;
    mpf_class ctemp;
    double dtemp;

    for (int incx = MAX_INCX; incx >= MIN_INCX; incx--) {
	for (int incy = MAX_INCY; incy >= MIN_INCY; incy--) {
	    for (int n = 0; n < MAX_N; n++) {
/*
		printf("# n:%d incx:%d, incy:%d dflag %lf vecx:%d vecy:%d\n",
		    n, incx, incy, dflag.get_d(), VEC_A(n, incx), VEC_A(n,
			incy));
*/
		mpf_class *x = new mpf_class[VEC_A(n, incx)];
		mpf_class *y = new mpf_class[VEC_A(n, incy)];
		mpf_class *dparam = new mpf_class[VEC_A(5, 1)];
		double *xd = new double[VEC_A(n, incx)];
		double *yd = new double[VEC_A(n, incy)];
		double *dparamd = new double[VEC_A(5, 1)];

		int j = 0;

		while (j < MAX_ITER) {
		    set_random_vector(x, xd, VEC_A(n, incx));
		    set_random_vector(y, yd, VEC_A(n, incy));
		    set_random_vector(dparam, dparamd, VEC_A(5, 1));
		    dparam[0] = dflag;
		    dparamd[0] = dflag.get_d();

		    Rrotm(n, x, incx, y, incy, dparam);
		    drotm_f77(&n, xd, &incx, yd, &incy, dparamd);

		    double diff;

		    diff = vec_diff(x, xd, VEC_A(n, incx), 1);
		    if (fabs(diff) > EPSILON) {
			printf("error in x %lf!!\n", diff);
			errorflag = TRUE;
		    }
		    diff = vec_diff(y, yd, VEC_A(n, incy), 1);
		    if (fabs(diff) > EPSILON) {
			printf("error in y %lf!!\n", diff);
			errorflag = TRUE;
		    }
		    diff = vec_diff(dparam, dparamd, 5, 1);
		    if (fabs(diff) > EPSILON) {
			printf("error in dparam %lf!!\n", diff);
			errorflag = TRUE;
		    }

		    j++;
		}
		delete[]dparam;
		delete[]dparamd;
		delete[]x;
		delete[]y;
		delete[]xd;
		delete[]yd;
	    }
	}
    }
    if (errorflag == TRUE) {
	printf("Rrotm test failed...\n");
	exit(-1);
    }
}

void
Rrotm_test()
{
    mpf_class dflag = 0;


    dflag = -1.0;
    Rrotm_test2(dflag);

    dflag = -2.0;
    Rrotm_test2(dflag);

    dflag = 0.0;
    Rrotm_test2(dflag);

    dflag = 1.0;
    Rrotm_test2(dflag);

}

int
main(int argc, char *argv[])
{
//initialization
    int default_prec = 200;
    gmp_randstate_t state;

    mpf_set_default_prec(default_prec);
    uniformrandom = new gmp_randclass(gmp_randinit_default);
    VERBOSE = 0;

    Rrotm_test();
    printf("Rrotm test passed...\n");

}
