/*************************************************************************
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 * Copyright 2008 by Nakata, Maho
 *
 * MPACK - multiple precision arithmetic library
 *
 * This file is part of MPACK.
 *
 * MPACK is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 3
 * only, as published by the Free Software Foundation.
 *
 * MPACK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3 for more details
 * (a copy is included in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3 along with MPACK.  If not, see
 * <http://www.gnu.org/licenses/lgpl.html>
 * for a copy of the LGPLv3 License.
 *
 ************************************************************************/
#include <stdio.h>
#include <iostream>
#include <fstream>
#include <cstdio>
#include <cstdlib>
#include <cmath>
#include <cstring>
#include <complex>

#include <mblas.h>
#include "blas.h"
#include "mblas.debug.h"

#define MIN_INCX   1
#define MAX_INCX   10
#define MIN_N      2
#define MAX_N      40
#define MAX_ITER 100

void
Rspr_test2(const char *uplo)
{
    int errorflag = FALSE;

    for (int incx = MIN_INCX; incx <= MAX_INCX; incx++) {
	for (int n = MIN_N; n < MAX_N; n++) {
//          printf("#n is %d, incx is %d, uplo is %s.\n", n, incx, uplo);
	    mpf_class alpha;
	    mpf_class *x;
	    mpf_class *AP;

	    double alphad;
	    double *xd;
	    double *APd;

	    x = new mpf_class[VEC_A(n, incx)];
	    AP = new mpf_class[VEC_A(n * (n + 1) / 2, 1)];
	    xd = new double[VEC_A(n, incx)];
	    APd = new double[VEC_A(n * (n + 1) / 2, 1)];

	    for (int i = 0; i < MAX_ITER; i++) {
		set_random_vector(AP, APd, VEC_A(n * (n + 1) / 2, 1));
		set_random_vector(x, xd, VEC_A(n, incx));
		set_random_number(&alpha, &alphad);

		Rspr(uplo, n, alpha, x, incx, AP);
		dspr_f77(uplo, &n, &alphad, xd, &incx, APd);

		double diff;

		diff = vec_diff(AP, APd, VEC_A(n * (n + 1) / 2, 1), 1);
		if (fabs(diff) > EPSILON) {
		    printf("#error %lf!!\n", diff);
		    errorflag = TRUE;
		}
	    }

	    delete[]APd;
	    delete[]xd;
	    delete[]AP;
	    delete[]x;
	}
    }
    if (errorflag == TRUE) {
	printf("Rspr test failed...\n");
	exit(-1);
    }

}

void
Rspr_test()
{
    Rspr_test2("U");
    Rspr_test2("L");
}

int
main(int argc, char *argv[])
{
//initialization
    int default_prec = 200;
    gmp_randstate_t state;

    mpf_set_default_prec(default_prec);
    uniformrandom = new gmp_randclass(gmp_randinit_default);
    VERBOSE = 0;

    Rspr_test();
    printf("Rspr test passed...\n");
}
