/*************************************************************************
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 * Copyright 2008 by Nakata, Maho
 *
 * MPACK - multiple precision arithmetic library
 *
 * This file is part of MPACK.
 *
 * MPACK is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 3
 * only, as published by the Free Software Foundation.
 *
 * MPACK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3 for more details
 * (a copy is included in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3 along with MPACK.  If not, see
 * <http://www.gnu.org/licenses/lgpl.html>
 * for a copy of the LGPLv3 License.
 *
 ************************************************************************/
#include <stdio.h>
#include <iostream>
#include <fstream>
#include <cstdio>
#include <cstdlib>
#include <cmath>
#include <cstring>
#include <complex>

#include <mblas.h>
#include "blas.h"
#include "mblas.debug.h"

#define MIN_INCX   1
#define MAX_INCX   10
#define MIN_INCY   1
#define MAX_INCY   10
#define MIN_N      2
#define MAX_N      20
#define MAX_LDA    20
#define MAX_ITER   30

void
Rsyr2_test2(const char *uplo)
{
    int errorflag = FALSE;

    for (int n = MIN_N; n < MAX_N; n++) {
	for (int lda = max(n, 1); lda < MAX_LDA; lda++) {
	    for (int incx = MIN_INCX; incx <= MAX_INCX; incx++) {
		for (int incy = MIN_INCY; incy < MAX_INCY; incy++) {
/*		    printf("#n is %d, incx is %d, uplo is %s lda is %d.\n", n,
			incx, uplo, lda); */
		    mpf_class alpha;
		    mpf_class *x;
		    mpf_class *y;
		    mpf_class *A;

		    double alphad;
		    double *xd;
		    double *yd;
		    double *Ad;

		    x = new mpf_class[VEC_A(n, incx)];
		    y = new mpf_class[VEC_A(n, incy)];
		    A = new mpf_class[MAT_A(lda, n)];
		    xd = new double[VEC_A(n, incx)];
		    yd = new double[VEC_A(n, incy)];
		    Ad = new double[MAT_A(lda, n)];

		    for (int i = 0; i < MAX_ITER; i++) {
			set_random_vector(A, Ad, MAT_A(lda, n));
			set_random_vector(x, xd, VEC_A(n, incx));
			set_random_vector(y, yd, VEC_A(n, incy));
			set_random_number(&alpha, &alphad);

			Rsyr2(uplo, n, alpha, x, incx, y, incy, A, lda);
			dsyr2_f77(uplo, &n, &alphad, xd, &incx, yd, &incy, Ad,
			    &lda);

			double diff;

			diff = vec_diff(A, Ad, MAT_A(lda, n), 1);
			if (fabs(diff) > EPSILON) {
			    printf("#error %lf!!\n", diff);
			    errorflag = TRUE;
			}
		    }

		    delete[]Ad;
		    delete[]xd;
		    delete[]A;
		    delete[]x;
		    delete[]y;
		    delete[]yd;
		}
	    }
	}
    }
    if (errorflag == TRUE) {
	printf("Rsyr2 test failed...\n");
	exit(-1);
    }
}

void
Rsyr2_test()
{
    Rsyr2_test2("U");
    Rsyr2_test2("L");
}

int
main(int argc, char *argv[])
{
//initialization
    int default_prec = 200;
    gmp_randstate_t state;

    mpf_set_default_prec(default_prec);
    uniformrandom = new gmp_randclass(gmp_randinit_default);
    VERBOSE = 0;

    Rsyr2_test();
    printf("Rsyr2 test passed...\n");
}
