/*************************************************************************
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 * Copyright 2008 by Nakata, Maho
 *
 * MPACK - multiple precision arithmetic library
 *
 * This file is part of MPACK.
 *
 * MPACK is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 3
 * only, as published by the Free Software Foundation.
 *
 * MPACK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3 for more details
 * (a copy is included in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3 along with MPACK.  If not, see
 * <http://www.gnu.org/licenses/lgpl.html>
 * for a copy of the LGPLv3 License.
 *
 ************************************************************************/
#include <stdio.h>
#include <iostream>
#include <fstream>
#include <cstdio>
#include <cstdlib>
#include <cmath>
#include <cstring>
#include <complex>

#include <mblas.h>
#include "blas.h"
#include "mblas.debug.h"

#define MIN_N -2
#define MAX_N 10
#define MIN_LDA -2
#define MAX_LDA 10
#define MIN_INCX -2
#define MAX_INCX 3
#define MAX_ITER 5

void
Rtrmv_test2(const char *uplo, const char *trans, const char *diag)
{
    int errorflag = FALSE;

    for (int incx = MIN_INCX; incx <= MAX_INCX; incx++) {
	for (int n = MIN_N; n < MAX_N; n++) {
	    for (int lda = max(1, n); lda < MAX_LDA; lda++) {
/*
		printf
		    ("#n is %d, lda is %d, incx is %d, uplo is %s, trans is %s, diag is %s \n",
		    n, lda, incx, uplo, trans, diag);
*/
		mpf_class *x;
		mpf_class *A;
		double *xd;
		double *Ad;

		A = new mpf_class[MAT_A(lda, n)];
		x = new mpf_class[VEC_A(n, incx)];
		Ad = new double[MAT_A(lda, n)];
		xd = new double[VEC_A(n, incx)];

		for (int i = 0; i < MAX_ITER; i++) {
		    set_random_vector(A, Ad, MAT_A(lda, n));
		    set_random_vector(x, xd, VEC_A(n, incx));

		    mpack_errno = 0;
		    Rtrmv(uplo, trans, diag, n, A, lda, x, incx);
		    dtrmv_f77(uplo, trans, diag, &n, Ad, &lda, xd, &incx);
		    if (mpack_errno != 0) {
			continue;
		    }

		    double diff;

		    if (Mlsame(trans, "N")) {
			diff = vec_diff(x, xd, (VEC_A(n, incx)), 1);
		    } else {
			diff = vec_diff(x, xd, (VEC_A(n, incx)), 1);
		    }
		    if (fabs(diff) > EPSILON) {
			printf("error %lf!!\n", diff);
			errorflag = TRUE;
		    } else {
//                      printf("#okay\n");
		    }
		}
		delete[]x;
		delete[]A;
		delete[]xd;
		delete[]Ad;
	    }
	}
    }

    if (errorflag == TRUE) {
	printf("Rtrmv test failed...\n");
	exit(-1);
    }
}

void
Rtrmv_test()
{
    Rtrmv_test2("U", "N", "U");
    Rtrmv_test2("U", "N", "N");
    Rtrmv_test2("U", "T", "U");
    Rtrmv_test2("U", "T", "N");
    Rtrmv_test2("U", "C", "U");
    Rtrmv_test2("U", "C", "N");

    Rtrmv_test2("L", "N", "U");
    Rtrmv_test2("L", "N", "N");
    Rtrmv_test2("L", "T", "U");
    Rtrmv_test2("L", "T", "N");
    Rtrmv_test2("L", "C", "U");
    Rtrmv_test2("L", "C", "N");

}

int
main(int argc, char *argv[])
{
//initialization
    int default_prec = 200;
    gmp_randstate_t state;

    mpf_set_default_prec(default_prec);
    uniformrandom = new gmp_randclass(gmp_randinit_default);
    VERBOSE = 0;

    Rtrmv_test();
    printf("Rtrmv test passed...\n");
}
