/*************************************************************************
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 * Copyright 2008 by Nakata, Maho
 *
 * MPACK - multiple precision arithmetic library
 *
 * This file is part of MPACK.
 *
 * MPACK is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 3
 * only, as published by the Free Software Foundation.
 *
 * MPACK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3 for more details
 * (a copy is included in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3 along with MPACK.  If not, see
 * <http://www.gnu.org/licenses/lgpl.html>
 * for a copy of the LGPLv3 License.
 *
 ************************************************************************/
#include <stdio.h>
#include <iostream>
#include <fstream>
#include <cstdio>
#include <cstdlib>
#include <cmath>
#include <cstring>
#include <complex>

#include <mblas.h>
#include "blas.h"
#include "mblas.debug.h"

#define MIN_INCX  1
#define MAX_INCX  2
#define MIN_N     1
#define MAX_N     10
#define MAX_ITER 10

void
iCamax_test()
{
    int errorflag = FALSE;

    int icmax;
    int izmax;

    for (int incx = MIN_INCX; incx <= MAX_INCX; incx++) {
	for (int n = MIN_N; n < MAX_N; n++) {
//          printf("#n:%d incx:%d \n", n, incx);
	    mpc_class *x = new mpc_class[VEC_A(n, incx)];
	    complex < double >*xd = new complex < double >[VEC_A(n, incx)];
	    int j = 0;

	    while (j < MAX_ITER) {
		set_random_vector(x, xd, VEC_A(n, incx));

		mpack_errno = 0;
		icmax = iCamax(n, x, incx);
		if (mpack_errno != 0) {
		    break;
		}
		izmax = izamax_f77(&n, xd, &incx);

		if (icmax != izmax) {
		    printf("error!!\n");
		    errorflag = TRUE;
		}
		j++;
	    }
	    delete[]x;
	    delete[]xd;
	}
    }

    if (errorflag == TRUE) {
	printf("iCamax test failed...\n");
	exit(-1);
    }
}

int
main(int argc, char *argv[])
{
//initialization
    int default_prec = 200;
    gmp_randstate_t state;

    mpf_set_default_prec(default_prec);
    uniformrandom = new gmp_randclass(gmp_randinit_default);


    iCamax_test();
    printf("iCamax test passed...\n");
}
