/*************************************************************************
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 * Copyright 2008 by Nakata, Maho
 *
 * MPACK - multiple precision arithmetic library
 *
 * This file is part of MPACK.
 *
 * MPACK is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 3
 * only, as published by the Free Software Foundation.
 *
 * MPACK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3 for more details
 * (a copy is included in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3 along with MPACK.  If not, see
 * <http://www.gnu.org/licenses/lgpl.html>
 * for a copy of the LGPLv3 License.
 *
 ************************************************************************/
/*
Copyright (c) 1992-2007 The University of Tennessee.  All rights reserved.

$COPYRIGHT$

Additional copyrights may follow

$HEADER$

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

- Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer. 
  
- Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer listed
  in this license in the documentation and/or other materials
  provided with the distribution.
  
- Neither the name of the copyright holders nor the names of its
  contributors may be used to endorse or promote products derived from
  this software without specific prior written permission.
  
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT  
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT  
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
*/

#include <mblas.h>
#include <mlapack.h>

void
Rsteqr(const char *compz, int n, mpf_class * d, mpf_class * e, mpf_class * Z,
    int ldz, mpf_class * work, int *info)
{
    int nmaxit, maxit, jtot, l1, nm1;
    int i, m, nm, mm, mm1, l, lm1, lend, lsv, lendsv, lendm1, iscale, icompz;

    mpf_class Zero, One, Two, Three;
    mpf_class eps, eps2, safmin, safmax, ssfmax, ssfmin;
    mpf_class c, s, rt1, rt2;
    mpf_class tst, anorm, po;
    mpf_class f, b, p, g, r;

    maxit = 30;
    Zero = 0.0;
    One = 1.0;
    Two = 2.0;
    Three = 3.0;

    *info = 0;

    if (Mlsame(compz, "N")) {
	icompz = 0;
    } else if (Mlsame(compz, "V")) {
	icompz = 1;
    } else if (Mlsame(compz, "I")) {
	icompz = 2;
    } else {
	icompz = -1;
    }
    if (icompz < 0) {
	*info = -1;
    } else if (n < 0) {
	*info = -2;
    } else if (ldz < 1 || icompz > 0 && ldz < max(1, n)) {
	*info = -6;
    }
    if (*info != 0) {
	Mxerbla("Rsteqr", -(*info));
	return;
    }
//Quick return if possible
    if (n == 0)
	return;

    if (n == 1) {
	if (icompz == 2)
	    Z[0] = One;
	return;
    }
//Determine the unit roundoff for this environment.
    eps = Rlamch("E");
    eps2 = eps * eps;
    safmin = Rlamch("S");
    safmax = One / safmin;
    ssfmax = sqrt(safmax) / Three;
    ssfmin = sqrt(safmin) / eps2;

//Compute the eigenvalues and eigenvectors of the tridiagonal matrix.
    if (icompz == 2)
	Rlaset("F", n, n, Zero, One, Z, ldz);
    nmaxit = n * maxit;
    jtot = 0;

//Determine where the matrix splits and choose QL or QR iteration
//for each block, according to whether top or bottom diagonal
//element is smaller.
    l1 = 0;
    nm1 = n - 2;

//
    while (jtot < nmaxit) {
	if (l1 > n - 1)
	    break;
	if (l1 > 0)
	    e[l1 - 1] = Zero;
	if (l1 <= nm1) {
	    for (m = l1; m < n - 1; m++) {
		tst = abs(e[m]);
		if (tst == Zero)
		    break;
		if (tst <= sqrt(abs(d[m])) * sqrt(abs(d[m + 1])) * eps) {
		    e[m] = Zero;
		    break;
		}
	    }
	}
	l = l1;
	lend = m;
	l1 = m + 1;
	lsv = l;
	lendsv = lend;
	if (lend == l)
	    continue;

//Scale submatrix in rows and columns L to LEND
//XXX not implemented yet.

	anorm = Rlanst("I", lend - l + 1, &d[l], &e[l]);
	iscale = 0;
	if (anorm > ssfmax) {
	    iscale = 1;
	    Rlascl("G", 0, 0, anorm, ssfmax, lend - l + 1, 1, &d[l], n, info);
	    Rlascl("G", 0, 0, anorm, ssfmax, lend - l, 1, &e[l], n, info);
	    printf("# Rsteqr: 1: XXX not very well tested\n");
	} else if (anorm < ssfmin) {
	    iscale = 2;
	    Rlascl("G", 0, 0, anorm, ssfmin, lend - l + 1, 1, &d[l], n, info);
	    Rlascl("G", 0, 0, anorm, ssfmin, lend - l, 1, &e[l], n, info);
	    printf("# Rsteqr: 2: XXX not very well tested\n");
	}
//Choose between QL and QR iteration
	if (abs(d[lend]) < abs(d[l])) {
	    lend = lsv;
	    l = lendsv;
	}
	if (lend >= l) {
//QL iteration
	    while (1) {
//look for small subdiagonal element.
		if (l != lend) {
		    for (m = l; m < lend; m++) {
			tst = abs(e[m] * e[m]);
			if (tst <= eps2 * abs(d[m] * e[m + 1]) + safmin)
			    break;
		    }
		} else {
		    m = lend;
		}
		if (m < lend)
		    e[m] = Zero;
		p = d[l];
		if (m != l) {
//If remaining matrix is 2-by-2, use DLAEV2 to compute its eigensystem.
		    if (m == l + 1) {
			Rlaev2(d[l], e[l], d[l + 1], &rt1, &rt2, &c, &s);
			work[l] = c;
			work[n - 1 + l] = s;
			Rlasr("R", "V", "B", n, 2, &work[l], &work[n - 1 + l],
			    &Z[l * ldz], ldz);
			d[l] = rt1;
			d[l + 1] = rt2;
			e[l] = Zero;
			l = l + 2;
			if (l <= lend)
			    continue;
			break;
		    }
		    if (jtot == nmaxit)
			continue;
		    jtot++;
//form shift
		    g = (d[l + 1] - p) / (Two * e[l]);
		    r = Rlapy2(g, One);
		    g = d[m] - p + (e[l] / (g + Msign(r, g)));
		    s = One;
		    c = One;
		    p = Zero;
//Inner loop
		    mm1 = m - 1;
		    for (i = mm1; i >= l; i--) {
			f = s * e[i];
			b = c * e[i];
			Rlartg(g, f, &c, &s, &r);
			if (i != m - 1)
			    e[i + 1] = r;
			g = d[i + 1] - p;
			r = (d[i] - g) * s + Two * c * b;
			p = s * r;
			d[i + 1] = g + p;
			g = c * r - b;
//If eigenvectors are desired, thus save rotations.
			if (icompz > 0) {
			    work[i] = c;
			    work[n - 1 + i] = -s;
			}
		    }
//Apply saved rotations;
		    if (icompz > 0) {
			mm = m - l + 1;
			Rlasr("R", "V", "B", n, mm, &work[l], &work[n - 1 + l], &Z[l * ldz], ldz);	//n should be ldz...
		    }
		    d[l] = d[l] - p;
		    e[l] = g;
		    continue;
		}
//Eigenvalue found
		d[l] = p;
		l++;
		if (l <= lend)
		    continue;
		break;
	    }
	} else {
//QR iteration
	    while (1) {
		if (l != lend) {
		    for (m = l; m >= lend + 1; m--) {
			tst = abs(e[m - 1] * e[m - 1]);
			if (tst <= eps2 * abs(d[m] * d[m - 1]) + safmin)
			    break;
		    }
		} else {
		    m = lend;
		}

		if (m > lend)
		    e[m - 1] = Zero;
		p = d[l];
		if (m != l) {
//If remaining matrix is 2-by-2, use DLAEV2 to compute its eigensystem.
		    if (m == l - 1) {
			Rlaev2(d[l - 1], e[l - 1], d[l], &rt1, &rt2, &c, &s);
			work[m] = c;
			work[n - 1 + m] = s;
			Rlasr("R", "V", "F", n, 2, &work[m], &work[n - 1 + m], &Z[(l - 1) * ldz], ldz);	//n should be ldz...
			d[l - 1] = rt1;
			d[l] = rt2;
			e[l - 1] = Zero;
			l = l - 2;
			if (l >= lend)
			    continue;
			break;
		    }
		    if (jtot == nmaxit)
			continue;
		    jtot++;
//form shift
		    g = (d[l - 1] - p) / (Two * e[l - 1]);
		    r = Rlapy2(g, One);
		    g = d[m] - p + (e[l - 1] / (g + Msign(r, g)));
		    s = One;

		    c = One;
		    p = Zero;
//Inner loop
		    lm1 = l - 1;
		    for (i = m; i <= lm1; i++) {
			f = s * e[i];
			b = c * e[i];
			Rlartg(g, f, &c, &s, &r);
			if (i != m)
			    e[i - 1] = r;
			g = d[i] - p;
			r = (d[i + 1] - g) * s + Two * c * b;
			p = s * r;
			d[i] = g + p;
			g = c * r - b;
//If eigenvectors are desired, save rotations.
			if (icompz > 0) {
			    work[i] = c;
			    work[n - 1 + i] = s;
			}
		    }
//Apply saved rotations;
		    if (icompz > 0) {
			mm = l - m + 1;
			Rlasr("R", "V", "B", n, mm, &work[m], &work[n - 1 + m], &Z[m * ldz], ldz);	//n should be ldz...
		    }
		    d[l] = d[l] - p;
		    e[lm1] = g;
		    continue;
		}
//Eigenvalue found
		d[l] = p;
		l--;
		if (l >= lend)
		    continue;
		break;
	    }

	}
//Undo scaling if necessary
	if (iscale == 1) {
	    Rlascl("G", 0, 0, ssfmax, anorm, lendsv - lsv + 1, 1, &d[lsv], n,
		info);
	    Rlascl("G", 0, 0, ssfmax, anorm, lendsv - lsv, 1, &e[lsv], n,
		info);
	    printf("# Rsteqr:3: XXX very well tested.\n");
	} else if (iscale == 2) {
	    Rlascl("G", 0, 0, ssfmin, anorm, lendsv - lsv + 1, 1, &d[lsv], n,
		info);
	    Rlascl("G", 0, 0, ssfmin, anorm, lendsv - lsv, 1, &e[lsv], n,
		info);
	    printf("# Rsteqr:4: XXX very well tested.\n");
	}
    }
    if (jtot > nmaxit) {
	for (int i = 0; i < n - 1; i++) {
	    if (e[i] != Zero)
		info++;
	}
    }
//All eigenvalues are calculated
//Use selection sort to minize swaps of eigenvectors
    if (icompz == 0) {
	Rlasrt("I", n, d, info);
    } else {
	for (int ii = 1; ii < n; ii++) {
	    int k, j;

	    i = ii - 1;
	    k = i;
	    p = d[i];
	    for (j = ii; j < n; j++) {
		if (d[j] < p) {
		    k = j;
		    p = d[j];
		}
	    }
	    if (k != i) {
		d[k] = d[i];
		d[i] = p;
		Rswap(n, &Z[i * ldz], 1, &Z[k * ldz], 1);
	    }
	}
    }
    return;
}
