/*************************************************************************
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 * Copyright 2008 by Nakata, Maho
 *
 * MPACK - multiple precision arithmetic library
 *
 * This file is part of MPACK.
 *
 * MPACK is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 3
 * only, as published by the Free Software Foundation.
 *
 * MPACK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3 for more details
 * (a copy is included in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3 along with MPACK.  If not, see
 * <http://www.gnu.org/licenses/lgpl.html>
 * for a copy of the LGPLv3 License.
 *
 ************************************************************************/
/*
Copyright (c) 1992-2007 The University of Tennessee.  All rights reserved.

$COPYRIGHT$

Additional copyrights may follow

$HEADER$

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

- Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer. 
  
- Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer listed
  in this license in the documentation and/or other materials
  provided with the distribution.
  
- Neither the name of the copyright holders nor the names of its
  contributors may be used to endorse or promote products derived from
  this software without specific prior written permission.
  
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT  
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT  
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
*/

#include <mblas.h>
#include <mlapack.h>

int
compare_mpf(const mpf_class * a, const mpf_class * b)
{
    if (*a > *b)
	return 1;
    if (*a == *b)
	return 0;
    if (*a < *b)
	return -1;

}

void
Rsterf(int n, mpf_class * d, mpf_class * e, int *info)
{
    mpf_class Zero, One, Two, Three;
    mpf_class sigma;
    mpf_class eps, eps2;
    mpf_class safmin, safmax, ssfmax, ssfmin, anorm;
    mpf_class rte, rt1, rt2, s, c, r, oldc, oldgam, gamma, p, bb, alpha;

    int maxit = 30;
    int nmaxit;
    int iscale;
    int l1, jtot;
    int i, l, m;
    int lsv, lend, lendsv;

    Zero = 0.0;
    One = 1.0;
    Two = 2.0;
    Three = 3.0;

    *info = 0;

//Quick return if possible
    if (n < 0) {
	*info = -1;
	Mxerbla("Rsterf", (*info));
    }
    if (n <= 1)
	return;

// Determine the unit roundoff for this environment.
    eps = Rlamch("E");
    eps2 = eps * eps;
    safmin = Rlamch("S");
    safmax = One / safmin;
    ssfmax = sqrt(safmax) / Three;
    ssfmin = sqrt(safmin) / eps2;

// Compute the eigenvalues of the tridiagonal matrix.
    nmaxit = n * maxit;
    sigma = Zero;
    jtot = 0;
    l1 = 0;
    lend = 0;
    l = 0;			//dummy values at least lend==l
    while (jtot < nmaxit) {
//Determine where the matrix splits and choose QL or QR iteration
//for each block, according to whether top or bottom diagonal
//element is smaller.
	if (l1 > n - 1)
	    break;
	if (l1 > 0)
	    e[l1 - 1] = Zero;
	for (m = l1; m < n - 1; m++) {
	    if (abs(e[m]) <= sqrt(abs(d[m])) * sqrt(abs(d[m + 1])) * eps) {
		e[m] = Zero;
		break;
	    }
	}
	l = l1;
	lend = m;
	l1 = m + 1;
	lsv = l;		//save l and lend in lsv and lendsv
	lendsv = lend;
//XXX NOT IMPLEMENTED
//Scale submatrix in rows and columns L to LEND
	anorm = Rlanst("I", lend - l + 1, &d[l], &e[l]);
	iscale = 0;
	if (anorm > ssfmax) {
	    iscale = 1;
	    printf("# Rsterf XXX rescale by ssfmax not implemented \n");
	    //Rlascl("G", &c0, &c0, &anorm, &ssfmax, lend - l + 1, &c1, &d__[l], n, info);
	    //Rlascl("G", &c0, &c0, &anorm, &ssfmax, lend-l, &c1, &e[l], n,   info);
	} else if (anorm < ssfmin) {
	    iscale = 2;
	    printf("# Rsterf XXX rescale by ssfmax not implemented \n");
	    //Rlascl("G", &c0, &c0, &anorm, &ssfmin, lend - l + 1, &c1, &d__[l], n, info);
	    //Rlascl("G", &c0, &c0, &anorm, &ssfmin, lend - l, &c1, &e[l], n, info);
	}
	for (i = l; i < lend; i++) {
	    e[i] = e[i] * e[i];
	}
//Choose between QL and QR iteration
	if (abs(d[lend]) < abs(d[l])) {
	    lend = lsv;
	    l = lendsv;
	}
	if (lend >= l) {
//QL iteration
	    while (1) {
//look for small subdiagonal element.
		if (l != lend) {
		    for (m = l; m < lend; m++) {
			if (abs(e[m]) <= eps2 * abs(d[m] * d[m + 1]))
			    break;
		    }
		} else {
		    m = lend;
		}
		if (m < lend)
		    e[m] = Zero;
		p = d[l];
		if (m != l) {
//if remaining matrix is 2 by 2, use dlae2 to compute its eigenvalues.
		    if (m == l + 1) {
			rte = sqrt(e[l]);
			Rlae2(d[l], rte, d[l + 1], &rt1, &rt2);
			d[l] = rt1;
			d[l + 1] = rt2;
			e[l] = Zero;
			l = l + 2;
			if (l <= lend)
			    continue;
			break;	//goto 150
		    }
		    if (jtot == nmaxit)
			break;
		    jtot++;
//form shift
		    rte = sqrt(e[l]);
		    sigma = (d[l + 1] - p) / (Two * rte);
		    r = Rlapy2(sigma, One);
		    sigma = p - (rte / (sigma + Msign(r, sigma)));
		    c = One;
		    s = Zero;
		    gamma = d[m] - sigma;
		    p = gamma * gamma;
//inner loop
		    for (i = m - 1; i >= l; i--) {
			bb = e[i];
			r = p + bb;
			if (i != m - 1)
			    e[i + 1] = s * r;
			oldc = c;
			c = p / r;
			s = bb / r;
			oldgam = gamma;
			alpha = d[i];
			gamma = c * (alpha - sigma) - s * oldgam;
			d[i + 1] = oldgam + (alpha - gamma);
			if (c != Zero)
			    p = (gamma * gamma) / c;
			else
			    p = oldc * bb;
		    }
		    e[l] = s * p;
		    d[l] = sigma + gamma;
		    continue;
		}
//Eigenvalue found.
		d[l] = p;
		l++;
		if (l <= lend)
		    continue;
		break;
	    }
	} else {
//QR Iteration
	    while (1) {
//look for small super diagonal element.
		if (l != lend) {
		    for (m = l; m >= lend + 1; m--) {
			if (abs(e[m - 1]) <= eps2 * abs(d[m] * d[m - 1]))
			    break;
		    }
		} else {
		    m = lend;
		}
		if (m > lend)
		    e[m - 1] = Zero;
		p = d[l];
		if (m != l) {
//if remaining matrix is 2 by 2, use dlae2 to compute its eigenvalues.
		    if (m == l - 1) {
			rte = sqrt(e[l - 1]);
			Rlae2(d[l], rte, d[l - 1], &rt1, &rt2);
			d[l] = rt1;
			d[l - 1] = rt2;
			e[l - 1] = Zero;
			l = l - 2;
			if (l >= lend)
			    continue;
			break;
		    }
		    if (jtot == nmaxit)
			break;
		    jtot++;
//form shift
		    rte = sqrt(e[l - 1]);
		    sigma = (d[l - 1] - p) / (Two * rte);
		    r = Rlapy2(sigma, One);
		    sigma = p - (rte / (sigma + Msign(r, sigma)));
		    c = One;
		    s = Zero;
		    gamma = d[m] - sigma;
		    p = gamma * gamma;
//inner loop
		    for (i = m; i <= l - 1; i++) {
			bb = e[i];
			r = p + bb;
			if (i != m)
			    e[i - 1] = s * r;
			oldc = c;
			c = p / r;
			s = bb / r;
			oldgam = gamma;
			alpha = d[i + 1];
			gamma = c * (alpha - sigma) - s * oldgam;
			d[i] = oldgam + (alpha - gamma);
			if (c != Zero)
			    p = (gamma * gamma) / c;
			else
			    p = oldc * bb;
		    }
		    e[l - 1] = s * p;
		    d[l] = sigma + gamma;
		    continue;
		}
//Eigenvalue found.
		d[l] = p;
		l--;
		if (l >= lend)
		    continue;
		break;
	    }
//Undo scaling if necessary
//XXX NOT IMPLEMENTED rescale matrix
	    if (iscale == 1) {
		printf("# Rsterf XXX rescale by ssfmax not implemented \n");
		//Rlascl("G", &c0, &c0, &ssfmax, &anorm, lendsv - lsv + 1, &c1, &d__[lsv], n, info);
	    } else if (iscale == 2) {
		printf("# Rsterf XXX rescale by ssfmin not implemented \n");
		//Rlascl("G", &c0, &c0, &ssfmin, &anorm, lendsv - lsv + 1, &c1, &d__[lsv], n, info);
	    }
	    jtot++;
	}
    }
    //Check for no convergence to an eigenvalue after a total
    //of N*MAXIT iterations.
    if (jtot > nmaxit) {
	for (int i = 0; i < n - 1; i++) {
	    if (e[i] != Zero)
		(*info)++;
	}
    }
    Rlasrt("I", n, d, info);
    return;
}
