/*************************************************************************
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 * Copyright 2008 by Nakata, Maho
 *
 * MPACK - multiple precision arithmetic library
 *
 * This file is part of MPACK.
 *
 * MPACK is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 3
 * only, as published by the Free Software Foundation.
 *
 * MPACK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3 for more details
 * (a copy is included in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3 along with MPACK.  If not, see
 * <http://www.gnu.org/licenses/lgpl.html>
 * for a copy of the LGPLv3 License.
 *
 ************************************************************************/

#include <cstdio>
#include <cstdlib>
#include <mblas.h>
#include <mlapack.h>

#include "blas.h"
#include "lapack.h"
#include "mblas.debug.h"

void
Rlae2_test()
{
    int errorflag = FALSE;
    mpf_class rtemp;
    double dtemp;

    mpf_class a, b, c, rt1, rt2;
    double ad, bd, cd, rt1d, rt2d;

    int count = 100;

    while (count--) {
	set_random_number(&a, &ad);
	set_random_number(&b, &bd);
	set_random_number(&c, &cd);
	Rlae2(a, b, c, &rt1, &rt2);
	dlae2_f77(&ad, &bd, &cd, &rt1d, &rt2d);
	if (num_diff(rt1, rt1d) > EPSILON) {
	    errorflag = TRUE;
	    printf("Error1\n");
	}
	if (num_diff(rt2, rt2d) > EPSILON) {
	    errorflag = TRUE;
	    printf("Error1\n");
	}

/* checking adf = ab (= O) case (|a-c| = 2|b|) */
	set_random_number(&a, &ad);
	set_random_number(&c, &cd);
	b = (a - c) * 0.5;
	bd = (ad - cd) * 0.5;
	Rlae2(a, b, c, &rt1, &rt2);
	dlae2_f77(&ad, &bd, &cd, &rt1d, &rt2d);
	if (num_diff(rt1, rt1d) > EPSILON) {
	    errorflag = TRUE;
	    printf("Error2\n");
	}
	if (num_diff(rt2, rt2d) > EPSILON) {
	    errorflag = TRUE;
	    printf("Error2\n");
	}

/* checking rt1=rt2 = 0 case */
	set_random_number(&a, &ad);
	set_random_number(&b, &bd);
	c = -a;
	cd = -ad;
	Rlae2(a, b, c, &rt1, &rt2);
	dlae2_f77(&ad, &bd, &cd, &rt1d, &rt2d);
	if (num_diff(rt1, rt1d) > EPSILON) {
	    errorflag = TRUE;
	    printf("Error3\n");
	}
	if (num_diff(rt2, rt2d) > EPSILON) {
	    errorflag = TRUE;
	    printf("Error3\n");
	}

	if (errorflag == TRUE) {
	    printf("Rlae2 test failed...\n");
	    exit(-1);
	}
    }
}

int
main(int argc, char *argv[])
{
//initialization
    int default_prec = 200;
    gmp_randstate_t state;

    mpf_set_default_prec(default_prec);
    uniformrandom = new gmp_randclass(gmp_randinit_default);

    Rlae2_test();
    printf("Rlae2 test passed...\n");
}
