#include <cstdio>
#include <cstdlib>

#include <mblas.h>
#include <mlapack.h>

#include "mblas.debug.h"
/* on FreeBSD 6/i386 + Lapack 3.1.1
  Epsilon                      =   1.110223024625157E-016
  Safe minimum                 =   2.225073858507201E-308
  Base                         =    2.00000000000000     
  Precision                    =   2.220446049250313E-016
  Number of digits in mantissa =    53.0000000000000     
  Rounding mode                =    1.00000000000000     
  Minimum exponent             =   -1021.00000000000     
  Underflow threshold          =   2.225073858507201E-308
  Largest exponent             =    1024.00000000000     
  Overflow threshold           =   1.797693134862316E+308
*/

void
Rlamch_test()
{
    mpf_class tmp, tmp2;

    tmp = Rlamch("E");
    tmp2 = Rlamch("E");
    gmp_printf("Rlamch E: Epsilon                      " GMP_P_FORMAT "\n",
	tmp.get_mpf_t());
    if (abs(tmp - tmp2) != 0.0) {
	printf("Errorin RlamchE \n");
	exit(-1);
    }
    if (1.0 + tmp <= 1.0) {
	printf("Errorin RlamchE \n");
	exit(-1);
    }

    tmp = Rlamch("S");
    gmp_printf("Rlamch S: Safe minimum                 " GMP_P_FORMAT "\n",
	tmp.get_mpf_t());
    tmp = Rlamch("B");
    gmp_printf("Rlamch B: Base                         " GMP_P_FORMAT "\n",
	tmp.get_mpf_t());
    tmp = Rlamch("P");
    gmp_printf("Rlamch P: Precision                    " GMP_P_FORMAT "\n",
	tmp.get_mpf_t());
    tmp = Rlamch("N");
    gmp_printf("Rlamch N: Number of digits in mantissa " GMP_P_FORMAT "\n",
	tmp.get_mpf_t());
    tmp = Rlamch("R");
    gmp_printf("Rlamch R: Rounding mode                " GMP_P_FORMAT "\n",
	tmp.get_mpf_t());
    tmp = Rlamch("M");
    gmp_printf("Rlamch M: Mimum exponent:              " GMP_P_FORMAT "\n",
	tmp.get_mpf_t());
    tmp = Rlamch("U");
    gmp_printf("Rlamch U: Underflow threshold          " GMP_P_FORMAT "\n",
	tmp.get_mpf_t());
    tmp = Rlamch("L");
    gmp_printf("Rlamch L: Largest exponent             " GMP_P_FORMAT "\n",
	tmp.get_mpf_t());
    tmp = Rlamch("O");
    gmp_printf("Rlamch O: Overflow threshold           " GMP_P_FORMAT "\n",
	tmp.get_mpf_t());
    tmp = Rlamch("S");
    tmp = 1.0 / tmp;
    gmp_printf("Rlamch -: Reciprocal of save minimum   " GMP_P_FORMAT "\n",
	tmp.get_mpf_t());
    tmp = Rlamch("Z");
    gmp_printf("Rlamch Z: dummy (error)                " GMP_P_FORMAT "\n",
	tmp.get_mpf_t());
}

int
main(int argc, char *argv[])
{
//initialization
    int default_prec = 200;
    gmp_randstate_t state;

    mpf_set_default_prec(default_prec);
    uniformrandom = new gmp_randclass(gmp_randinit_default);

    Rlamch_test();
    printf("All tests passed succesfully\n");
}
