/*************************************************************************
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 * Copyright 2008 by Nakata, Maho
 *
 * MPACK - multiple precision arithmetic library
 *
 * This file is part of MPACK.
 *
 * MPACK is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 3
 * only, as published by the Free Software Foundation.
 *
 * MPACK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3 for more details
 * (a copy is included in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3 along with MPACK.  If not, see
 * <http://www.gnu.org/licenses/lgpl.html>
 * for a copy of the LGPLv3 License.
 *
 ************************************************************************/

#include <cstdio>
#include <cstdlib>
#include <mblas.h>
#include <mlapack.h>

#include "blas.h"
#include "lapack.h"
#include "mblas.debug.h"

#define MIN_N     0
#define MAX_N     100
#define MAX_ITER  10

void Rlanst_test2(const char *norm);

void
Rlanst_test2(const char *norm)
{
    int errorflag = FALSE;
    int j = 0;
    mpf_class Rlanst_ret;
    double dlanst_ret;

    for (int n = MIN_N; n < MAX_N; n++) {
	printf("n:%d norm %s\n", n, norm);
	mpf_class *d = new mpf_class[VEC_A(n, 1)];
	mpf_class *e = new mpf_class[VEC_A(n - 1, 1)];

	double *dd = new double[VEC_A(n, 1)];
	double *ed = new double[VEC_A(n - 1, 1)];

	j = 0;
	while (j < MAX_ITER) {
	    set_random_vector(d, dd, VEC_A(n, 1));
	    set_random_vector(e, ed, VEC_A(n - 1, 1));

	    Rlanst_ret = Rlanst(norm, n, d, e);
	    dlanst_ret = dlanst_f77(norm, &n, dd, ed);

	    double diff = num_diff(Rlanst_ret, dlanst_ret);

	    if (fabs(diff) > EPSILON) {
		printf("error %lf!!\n", diff);
		errorflag = TRUE;
	    }
	    j++;
	}
	delete[]d;
	delete[]dd;
	delete[]e;
	delete[]ed;
    }
    if (errorflag == TRUE) {
	printf("Rlanst test failed...\n");
	exit(-1);
    }
}

void
Rlanst_test(void)
{
    Rlanst_test2("M");
    Rlanst_test2("m");
    Rlanst_test2("1");
    Rlanst_test2("O");
    Rlanst_test2("o");
    Rlanst_test2("I");
    Rlanst_test2("i");
    Rlanst_test2("F");
    Rlanst_test2("f");
    Rlanst_test2("E");
    Rlanst_test2("e");
}

int
main(int argc, char *argv[])
{
//initialization
    int default_prec = 200;
    gmp_randstate_t state;

    mpf_set_default_prec(default_prec);
    uniformrandom = new gmp_randclass(gmp_randinit_default);

    Rlanst_test();
    printf("Rlanst test passed...\n");

}
