/*************************************************************************
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 * Copyright 2008 by Nakata, Maho
 *
 * MPACK - multiple precision arithmetic library
 *
 * This file is part of MPACK.
 *
 * MPACK is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 3
 * only, as published by the Free Software Foundation.
 *
 * MPACK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3 for more details
 * (a copy is included in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3 along with MPACK.  If not, see
 * <http://www.gnu.org/licenses/lgpl.html>
 * for a copy of the LGPLv3 License.
 *
 ************************************************************************/

#include <cstdio>
#include <cstdlib>
#include <mblas.h>
#include <mlapack.h>

#include "blas.h"
#include "lapack.h"
#include "mblas.debug.h"

#define MIN_N     0
#define MAX_N     50
#define MAX_LDA   60
#define MAX_ITER  10

void Rlansy_test2(const char *norm, const char *uplo);

void
Rlansy_test2(const char *norm, const char *uplo)
{
    int errorflag = FALSE;
    int j = 0;
    mpf_class Rlansy_ret;
    double dlansy_ret;

    for (int n = MIN_N; n < MAX_N; n++) {
	for (int lda = max(n, 1); lda < MAX_LDA; lda++) {
	    mpf_class *A = new mpf_class[MAT_A(lda, n)];
	    mpf_class *work = new mpf_class[max(1, n)];

	    double *Ad = new double[MAT_A(lda, n)];
	    double *workd = new double[max(1, n)];

	    j = 0;
	    while (j < MAX_ITER) {
		set_random_vector(A, Ad, MAT_A(lda, n));
		Rlansy_ret = Rlansy(norm, uplo, n, A, lda, work);
		dlansy_ret = dlansy_f77(norm, uplo, &n, Ad, &lda, workd);

		double diff = num_diff(Rlansy_ret, dlansy_ret);

		if (fabs(diff) > EPSILON) {
		    printf("n:%d lda %d, uplo %s, norm %s\n", n, lda, uplo,
			norm);
		    printf("error %.16lf!!\n", diff);
		    errorflag = TRUE;
		}
		j++;
	    }
	    delete[]A;
	    delete[]Ad;
	    delete[]work;
	    delete[]workd;
	}
	if (errorflag == TRUE) {
	    printf("Rlansy test failed...\n");
	    exit(-1);
	}
    }
}

void
Rlansy_test(void)
{
    Rlansy_test2("M", "U");
    Rlansy_test2("m", "U");
    Rlansy_test2("1", "U");
    Rlansy_test2("O", "U");
    Rlansy_test2("o", "U");
    Rlansy_test2("I", "U");
    Rlansy_test2("i", "U");
    Rlansy_test2("F", "U");
    Rlansy_test2("f", "U");
    Rlansy_test2("E", "U");
    Rlansy_test2("e", "U");

    Rlansy_test2("M", "L");
    Rlansy_test2("m", "L");
    Rlansy_test2("1", "L");
    Rlansy_test2("O", "L");
    Rlansy_test2("o", "L");
    Rlansy_test2("I", "L");
    Rlansy_test2("i", "L");
    Rlansy_test2("F", "L");
    Rlansy_test2("f", "L");
    Rlansy_test2("E", "L");
    Rlansy_test2("e", "L");
}

int
main(int argc, char *argv[])
{
//initialization
    int default_prec = 200;
    gmp_randstate_t state;

    mpf_set_default_prec(default_prec);
    uniformrandom = new gmp_randclass(gmp_randinit_default);

    Rlansy_test();
    printf("Rlansy test passed...\n");

}
