/*************************************************************************
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 * Copyright 2008 by Nakata, Maho
 *
 * MPACK - multiple precision arithmetic library
 *
 * This file is part of MPACK.
 *
 * MPACK is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 3
 * only, as published by the Free Software Foundation.
 *
 * MPACK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3 for more details
 * (a copy is included in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3 along with MPACK.  If not, see
 * <http://www.gnu.org/licenses/lgpl.html>
 * for a copy of the LGPLv3 License.
 *
 ************************************************************************/

#include <cstdio>
#include <cstdlib>
#include <mblas.h>
#include <mlapack.h>

#include "blas.h"
#include "lapack.h"
#include "mblas.debug.h"

void
Rlapy2_test()
{
    int errorflag = FALSE;
    mpf_class rtemp;
    double dtemp;

    mpf_class Rlapy2_ret, x, y;
    double dlapy2_ret, xd, yd;

    int count = 100;

    while (count--) {
	set_random_number(&x, &xd);
	set_random_number(&y, &yd);

	Rlapy2_ret = Rlapy2(x, y);
	dlapy2_ret = dlapy2_f77(&xd, &yd);
	gmp_printf(GMP_P_FORMAT "\n", x.get_mpf_t());
	gmp_printf(GMP_P_FORMAT "\n", y.get_mpf_t());
	gmp_printf(GMP_P_FORMAT "\n", Rlapy2_ret.get_mpf_t());

	printf(P_FORMAT "\n", xd);
	printf(P_FORMAT "\n", yd);
	printf(P_FORMAT "\n", dlapy2_ret);

	if (num_diff(Rlapy2_ret, dlapy2_ret) > EPSILON) {
	    errorflag = TRUE;
	    printf("Error1\n");
	}
    }
    if (errorflag == TRUE) {
	printf("Rlaev2 test failed...\n");
	exit(-1);
    }
}

int
main(int argc, char *argv[])
{
//initialization
    int default_prec = 200;
    gmp_randstate_t state;

    mpf_set_default_prec(default_prec);
    uniformrandom = new gmp_randclass(gmp_randinit_default);

    Rlapy2_test();
    printf("Rlaev2 test passed...\n");
}
