/*************************************************************************
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 * Copyright 2008 by Nakata, Maho
 *
 * MPACK - multiple precision arithmetic library
 *
 * This file is part of MPACK.
 *
 * MPACK is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 3
 * only, as published by the Free Software Foundation.
 *
 * MPACK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3 for more details
 * (a copy is included in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3 along with MPACK.  If not, see
 * <http://www.gnu.org/licenses/lgpl.html>
 * for a copy of the LGPLv3 License.
 *
 ************************************************************************/

#include <cstdio>
#include <cstdlib>
#include <mblas.h>
#include <mlapack.h>

#include "blas.h"
#include "lapack.h"
#include "mblas.debug.h"

#define MIN_N 2
#define MAX_N 20
#define MIN_M 2
#define MAX_M 20
#define MIN_LDC 2
#define MAX_LDC 20
#define MAX_ITER 10
#define MIN_INCV -3
#define MAX_INCV  3

void
Rlarf_test2(const char *side)
{
    int errorflag = FALSE;
    int dimv;
    int n, m, ldc, incv, iter;
    mpf_class tau;
    mpf_class *C, *v, *work;
    double taud;
    double *Cd, *vd, *workd;

    for (incv = MIN_INCV; incv <= MAX_INCV; incv++) {
	if (incv == 0)
	    continue;
	for (n = MIN_N; n <= MAX_N; n++) {
	    for (m = MIN_M; m <= MAX_M; m++) {
		for (ldc = max(1, m); ldc <= MAX_LDC; ldc++) {

		    printf("# m: %d, n: %d, incv: %d, ldc: %d, side %s\n", m,
			n, incv, ldc, side);
		    if (Mlsame(side, "L"))
			dimv = (1 + (m - 1) * abs(incv));
		    if (Mlsame(side, "R"))
			dimv = (1 + (n - 1) * abs(incv));

		    C = new mpf_class[MAT_A(ldc, n)];
		    v = new mpf_class[dimv];
		    if (Mlsame(side, "L"))
			work = new mpf_class[n];
		    if (Mlsame(side, "R"))
			work = new mpf_class[m];

		    Cd = new double[MAT_A(ldc, n)];
		    vd = new double[dimv];

		    if (Mlsame(side, "L"))
			workd = new double[n];

		    if (Mlsame(side, "R"))
			workd = new double[m];

		    for (iter = 0; iter < MAX_ITER; iter++) {
			set_random_vector(C, Cd, MAT_A(ldc, n));
			set_random_vector(v, vd, dimv);
			set_random_number(&tau, &taud);

			Rlarf(side, m, n, v, incv, tau, C, ldc, work);
			dlarf_f77(side, &m, &n, vd, &incv, &taud, Cd, &ldc,
			    workd);

			double diff;

			diff = vec_diff(C, Cd, MAT_A(ldc, n), 1);
			printf("#error %.16lf!!\n", diff);
			if (fabs(diff) > EPSILON) {
			    printf("#error %.16lf!!\n", diff);
			    errorflag = TRUE;
			} else {
//printf("#okay\n");
			}
		    }
		    delete[]vd;
		    delete[]Cd;
		    delete[]v;
		    delete[]C;

		}
	    }

	}
    }
    if (errorflag == TRUE) {
	printf("Rlarf test failed...\n");
	exit(-1);
    }
}

void
Rlarf_test()
{

    Rlarf_test2("L");
    Rlarf_test2("R");

}

int
main(int argc, char *argv[])
{
//initialization
    int default_prec = 100;
    gmp_randstate_t state;

    mpf_set_default_prec(default_prec);
    uniformrandom = new gmp_randclass(gmp_randinit_default);
    VERBOSE = 0;

    Rlarf_test();
    printf("Rlarf test passed...\n");

}
