/*************************************************************************
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 * Copyright 2008 by Nakata, Maho
 *
 * MPACK - multiple precision arithmetic library
 *
 * This file is part of MPACK.
 *
 * MPACK is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 3
 * only, as published by the Free Software Foundation.
 *
 * MPACK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3 for more details
 * (a copy is included in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3 along with MPACK.  If not, see
 * <http://www.gnu.org/licenses/lgpl.html>
 * for a copy of the LGPLv3 License.
 *
 ************************************************************************/

#include <cstdio>
#include <cstdlib>
#include <mblas.h>
#include <mlapack.h>

#include "blas.h"
#include "lapack.h"
#include "mblas.debug.h"

#define MIN_K 1
#define MAX_K 10
#define MIN_N 1
#define MAX_N 10
#define MIN_M 1
#define MAX_M 10
#define MAX_LDT 12
#define MAX_LDC 12
#define MAX_LDV 12
#define MAX_ITER 10

void
Rlarfb_test2(const char *side, const char *trans, const char *direct,
    const char *storev)
{
    int errorflag = FALSE;
    int iter;
    int k, m, n;
    int ldt, ldc;
    int ldv, ldvmin, v_column, ldwork, kmax;

    for (n = MIN_N; n <= MAX_N; n++) {
	for (m = MIN_M; m <= MAX_M; m++) {
	    if (Mlsame(side, "L"))
		kmax = m;
	    if (Mlsame(side, "R"))
		kmax = n;
	    for (k = MIN_K; k <= kmax; k++) {

		for (ldt = k; ldt <= MAX_LDT; ldt++) {
		    for (ldc = max(1, m); ldc <= MAX_LDC; ldc++) {
			if (Mlsame(storev, "C") && Mlsame(side, "L"))
			    ldvmin = max(1, m);
			if (Mlsame(storev, "C") && Mlsame(side, "R"))
			    ldvmin = max(1, n);
			if (Mlsame(storev, "R"))
			    ldvmin = k;

			for (ldv = ldvmin; ldv <= MAX_LDV; ldv++) {

			    if (Mlsame(side, "L"))
				ldwork = max(1, n);
			    if (Mlsame(side, "R"))
				ldwork = max(1, m);
			    if (Mlsame(storev, "C"))
				v_column = k;
			    if (Mlsame(storev, "R") && Mlsame(side, "L"))
				v_column = m;
			    if (Mlsame(storev, "R") && Mlsame(side, "R"))
				v_column = n;

			    mpf_class *T = new mpf_class[MAT_A(ldt, k)];
			    mpf_class *C = new mpf_class[MAT_A(ldc, n)];

			    mpf_class *V = new mpf_class[MAT_A(ldv, v_column)];
			    mpf_class *work = new mpf_class[MAT_A(ldwork, k)];

			    double *Td = new double[MAT_A(ldt, k)];
			    double *Cd = new double[MAT_A(ldc, n)];
			    double *Vd = new double[MAT_A(ldv, v_column)];
			    double *workd = new double[MAT_A(ldwork, k)];

			    printf
				("# side %s: trans %s: direct %s: storev %s, m %d, n %d, k %d, ldv %d, ldt %d, ldc %d, ldwork %d\n",
				side, trans, direct, storev, m, n, k, ldv, ldt,
				ldc, ldwork);

			    for (iter = 0; iter < MAX_ITER; iter++) {
				set_random_vector(T, Td, MAT_A(ldt, k));
				set_random_vector(C, Cd, MAT_A(ldc, n));
				set_random_vector(V, Vd, MAT_A(ldv, v_column));
				dlarfb_f77(side, trans, direct, storev, &m, &n,
				    &k, Vd, &ldv, Td, &ldt, Cd, &ldc, workd,
				    &ldwork);
				Rlarfb(side, trans, direct, storev, m, n, k, V,
				    ldv, T, ldt, C, ldc, work, ldwork);

				double diff;

				diff = vec_diff(C, Cd, MAT_A(ldc, n), 1);
//              printf("#error %e!\n", diff);
				if (fabs(diff) > EPSILON) {
				    printf("#error %e!!\n", diff);
				    errorflag = TRUE;
				} else {
//        printf("#okay\n");
				}
			    }
			    delete[]workd;
			    delete[]Vd;
			    delete[]Cd;
			    delete[]Td;
			    delete[]work;
			    delete[]V;
			    delete[]C;
			    delete[]T;
			}
		    }
		}
	    }
	}
    }
    if (errorflag == TRUE) {
	printf("Rlarfb test failed...\n");
	exit(-1);
    }
}

void
Rlarfb_test()
{

    Rlarfb_test2("L", "N", "F", "C");
    Rlarfb_test2("L", "T", "F", "C");
    Rlarfb_test2("L", "N", "B", "C");
    Rlarfb_test2("L", "T", "B", "C");
    Rlarfb_test2("R", "N", "F", "C");
    Rlarfb_test2("R", "T", "F", "C");
    Rlarfb_test2("R", "N", "B", "C");
    Rlarfb_test2("R", "T", "B", "C");

    Rlarfb_test2("L", "N", "F", "R");
    Rlarfb_test2("L", "T", "F", "R");
    Rlarfb_test2("L", "N", "B", "R");
    Rlarfb_test2("L", "T", "B", "R");
    Rlarfb_test2("R", "N", "F", "R");
    Rlarfb_test2("R", "T", "F", "R");
    Rlarfb_test2("R", "N", "B", "R");
    Rlarfb_test2("R", "T", "B", "R");
}

int
main(int argc, char *argv[])
{
//initialization
    int default_prec = 100;
    gmp_randstate_t state;

    mpf_set_default_prec(default_prec);
    uniformrandom = new gmp_randclass(gmp_randinit_default);
    VERBOSE = 0;

    Rlarfb_test();
    printf("Rlarfb test passed...\n");

}
