/*************************************************************************
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 * Copyright 2008 by Nakata, Maho
 *
 * MPACK - multiple precision arithmetic library
 *
 * This file is part of MPACK.
 *
 * MPACK is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 3
 * only, as published by the Free Software Foundation.
 *
 * MPACK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3 for more details
 * (a copy is included in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3 along with MPACK.  If not, see
 * <http://www.gnu.org/licenses/lgpl.html>
 * for a copy of the LGPLv3 License.
 *
 ************************************************************************/

#include <cstdio>
#include <cstdlib>
#include <mblas.h>
#include <mlapack.h>

#include "blas.h"
#include "lapack.h"
#include "mblas.debug.h"

#define MIN_N 2
#define MAX_N 20
#define MIN_INCX  1
#define MAX_INCX  3
#define MAX_ITER 10

void
Rlarfg_test()
{
    int errorflag = FALSE;
    int incx, n, iter;
    mpf_class alpha, tau;
    double alphad, taud;
    mpf_class *x;
    double *xd;

    for (incx = MIN_INCX; incx <= MAX_INCX; incx++) {
	for (n = MIN_N; n <= MAX_N; n++) {
	    printf("# n: %d, incx: %d \n", n, incx);

	    x = new mpf_class[VEC_A(n, incx)];
	    xd = new double[VEC_A(n, incx)];

	    for (iter = 0; iter < MAX_ITER; iter++) {
		set_random_vector(x, xd, VEC_A(n, incx));
		set_random_number(&alpha, &alphad);

		Rlarfg(n, &alpha, x, incx, &tau);
		dlarfg_f77(&n, &alphad, xd, &incx, &taud);

		double diff;

		diff = num_diff(alpha, alphad);

		if (fabs(diff) > EPSILON) {
		    printf("#error alpha %.16lf!!\n", diff);
		    errorflag = TRUE;
		}

		diff = num_diff(tau, taud);
		if (fabs(diff) > EPSILON) {
		    printf("#error tau %.16lf!!\n", diff);
		    errorflag = TRUE;
		}

	    }

	    delete[]xd;
	    delete[]x;
	}
    }


    if (errorflag == TRUE) {
	printf("Rlarfg test failed...\n");
	exit(-1);
    }
}

int
main(int argc, char *argv[])
{
//initialization
    int default_prec = 100;
    gmp_randstate_t state;

    mpf_set_default_prec(default_prec);
    uniformrandom = new gmp_randclass(gmp_randinit_default);
    VERBOSE = 0;

    Rlarfg_test();
    printf("Rlarfg test passed...\n");

}
