/*************************************************************************
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 * Copyright 2008 by Nakata, Maho
 *
 * MPACK - multiple precision arithmetic library
 *
 * This file is part of MPACK.
 *
 * MPACK is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 3
 * only, as published by the Free Software Foundation.
 *
 * MPACK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3 for more details
 * (a copy is included in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3 along with MPACK.  If not, see
 * <http://www.gnu.org/licenses/lgpl.html>
 * for a copy of the LGPLv3 License.
 *
 ************************************************************************/

#include <cstdio>
#include <cstdlib>
#include <mblas.h>
#include <mlapack.h>

#include "blas.h"
#include "lapack.h"
#include "mblas.debug.h"

#define MIN_K 1
#define MAX_K 10
#define MIN_N 0
#define MAX_N 10
#define MAX_LDT 12
#define MAX_LDC 12
#define MAX_LDV 12
#define MAX_ITER 10

void
Rlarft_test2(const char *direct, const char *storev)
{
    int errorflag = FALSE;
    int n, k, ldvmin, v_column, ldt, iter, ldv, nmin;

    for (k = MIN_K; k <= MAX_K; k++) {
	for (n = k; n <= MAX_N; n++) {
	    for (ldt = k; ldt <= MAX_LDT; ldt++) {
		if (Mlsame(storev, "C"))
		    ldvmin = max(1, n);
		if (Mlsame(storev, "R"))
		    ldvmin = max(1, k);

		for (ldv = ldvmin; ldv <= MAX_LDV; ldv++) {

		    if (Mlsame(storev, "C"))
			v_column = k;
		    if (Mlsame(storev, "R"))
			v_column = n;

		    mpf_class *T = new mpf_class[MAT_A(ldt, k)];
		    mpf_class *V = new mpf_class[MAT_A(ldv, v_column)];
		    mpf_class *tau = new mpf_class[VEC_A(k, 1)];

		    double *Td = new double[MAT_A(ldt, k)];
		    double *Vd = new double[MAT_A(ldv, v_column)];
		    double *taud = new double[VEC_A(k, 1)];

		    printf
			("#direct %s: storev %s, n %d, k %d, ldv %d, ldt %d\n",
			direct, storev, n, k, ldv, ldt);

		    for (iter = 0; iter < MAX_ITER; iter++) {
			set_random_vector(T, Td, MAT_A(ldt, k));
			set_random_vector(V, Vd, MAT_A(ldv, v_column));
			set_random_vector(tau, taud, VEC_A(k, 1));

			Rlarft(direct, storev, n, k, V, ldv, tau, T, ldt);
			dlarft_f77(direct, storev, &n, &k, Vd, &ldv, taud, Td,
			    &ldt);

			double diff;

			diff = vec_diff(T, Td, MAT_A(ldt, k), 1);
			if (fabs(diff) > EPSILON) {
			    printf("#error %e!!\n", diff);
			    errorflag = TRUE;
			} else {
//        printf("#okay\n");
			}
		    }
		    delete[]taud;
		    delete[]Vd;
		    delete[]Td;
		    delete[]tau;
		    delete[]V;
		    delete[]T;
		}
	    }
	}
    }
    if (errorflag == TRUE) {
	printf("Rlarft test failed...\n");
	exit(-1);
    }
}

void
Rlarft_test()
{
    Rlarft_test2("F", "C");
    Rlarft_test2("F", "R");
    Rlarft_test2("B", "C");
    Rlarft_test2("B", "R");
}

int
main(int argc, char *argv[])
{
//initialization
    int default_prec = 100;
    gmp_randstate_t state;

    mpf_set_default_prec(default_prec);
    uniformrandom = new gmp_randclass(gmp_randinit_default);
    VERBOSE = 0;

    Rlarft_test();
    printf("Rlarft test passed...\n");

}
