/*************************************************************************
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 * Copyright 2008 by Nakata, Maho
 *
 * MPACK - multiple precision arithmetic library
 *
 * This file is part of MPACK.
 *
 * MPACK is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 3
 * only, as published by the Free Software Foundation.
 *
 * MPACK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3 for more details
 * (a copy is included in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3 along with MPACK.  If not, see
 * <http://www.gnu.org/licenses/lgpl.html>
 * for a copy of the LGPLv3 License.
 *
 ************************************************************************/

#include <cstdio>
#include <cstdlib>
#include <mblas.h>
#include <mlapack.h>

#include "blas.h"
#include "lapack.h"
#include "mblas.debug.h"

void
Rlartg_test()
{
    int errorflag = FALSE;
    mpf_class rtemp;
    double dtemp;

    mpf_class f, g, cs, sn, r;
    double fd, gd, csd, snd, rd;

    int count = 100;

    while (count--) {
	set_random_number(&f, &fd);
	set_random_number(&g, &gd);
	Rlartg(f, g, &cs, &sn, &r);
	dlartg_f77(&fd, &gd, &csd, &snd, &rd);

	if (num_diff(cs, csd) > EPSILON) {
	    errorflag = TRUE;
	    printf("Error1\n");
	}
	if (num_diff(sn, snd) > EPSILON) {
	    errorflag = TRUE;
	    printf("Error1\n");
	}

	if (num_diff(r, rd) > EPSILON) {
	    errorflag = TRUE;
	    printf("Error1\n");
	}


	f = 0.0;
	fd = 0.0;
	set_random_number(&g, &gd);
	Rlartg(f, g, &cs, &sn, &r);
	dlartg_f77(&fd, &gd, &csd, &snd, &rd);

	if (num_diff(cs, csd) > EPSILON) {
	    errorflag = TRUE;
	    printf("Error1\n");
	}
	if (num_diff(sn, snd) > EPSILON) {
	    errorflag = TRUE;
	    printf("Error1\n");
	}

	if (num_diff(r, rd) > EPSILON) {
	    errorflag = TRUE;
	    printf("Error1\n");
	}


	g = 0.0;
	gd = 0.0;
	set_random_number(&f, &fd);
	Rlartg(f, g, &cs, &sn, &r);
	dlartg_f77(&fd, &gd, &csd, &snd, &rd);

	if (num_diff(cs, csd) > EPSILON) {
	    errorflag = TRUE;
	    printf("Error1\n");
	}
	if (num_diff(sn, snd) > EPSILON) {
	    errorflag = TRUE;
	    printf("Error1\n");
	}

	if (num_diff(r, rd) > EPSILON) {
	    errorflag = TRUE;
	    printf("Error1\n");
	}



	if (errorflag == TRUE) {
	    printf("Rlartg test failed...\n");
	    exit(-1);
	}
    }
}

int
main(int argc, char *argv[])
{
//initialization
    int default_prec = 200;
    gmp_randstate_t state;

    mpf_set_default_prec(default_prec);
    uniformrandom = new gmp_randclass(gmp_randinit_default);

    Rlartg_test();
    printf("Rlartg test passed...\n");
}
