/*************************************************************************
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 * Copyright 2008 by Nakata, Maho
 *
 * MPACK - multiple precision arithmetic library
 *
 * This file is part of MPACK.
 *
 * MPACK is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 3
 * only, as published by the Free Software Foundation.
 *
 * MPACK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3 for more details
 * (a copy is included in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3 along with MPACK.  If not, see
 * <http://www.gnu.org/licenses/lgpl.html>
 * for a copy of the LGPLv3 License.
 *
 ************************************************************************/

#include <cstdio>
#include <cstdlib>
#include <mblas.h>
#include <mlapack.h>

#include "blas.h"
#include "lapack.h"
#include "mblas.debug.h"

#define MIN_N     5
#define MAX_N     10
#define MIN_M     5
#define MAX_M     10
#define MIN_LDA   5
#define MAX_LDA   10
#define MAX_ITER  10

void Rlascl_test2(const char *type);

void
Rlascl_test2(const char *type)
{
    int errorflag = FALSE;
    int j, m, n, lda, ku, kl, info, infod;
    mpf_class cto, cfrom;
    double ctod, cfromd;

    for (n = MIN_N; n <= MAX_N; n++) {
	for (m = MIN_M; m <= MAX_M; m++) {
	    for (lda = m; lda <= MAX_LDA; lda++) {
		for (kl = 0; kl <= n; kl++) {
		    for (ku = 0; ku <= m; ku++) {
			mpf_class *A = new mpf_class[MAT_A(lda, n)];
			double *Ad = new double[MAT_A(lda, n)];

			printf("#type %s, n %d, m %d, lda %d, ku %d, kl %d\n",
			    type, n, m, lda, ku, kl);
			j = 0;
			while (j < MAX_ITER) {
			    set_random_vector(A, Ad, MAT_A(lda, n));
			    set_random_number(&cto, &ctod);
			    set_random_number(&cfrom, &cfromd);

			    dlascl_f77(type, &kl, &ku, &cfromd, &ctod, &m, &n,
				Ad, &lda, &infod);
			    Rlascl(type, kl, ku, cfrom, cto, m, n, A, lda,
				&info);

			    if (info != infod) {
				printf("error in info %d %d!!\n", info, infod);
				errorflag = TRUE;
			    }

			    double diff = vec_diff(A, Ad, MAT_A(lda, n), 1);

			    if (fabs(diff) > EPSILON2) {
				printf("error %e!!\n", diff);
				errorflag = TRUE;
				exit(-1);
			    }
			    j++;
			}
			delete[]Ad;
			delete[]A;
		    }
		}
	    }
	}
    }

    if (errorflag == TRUE) {
	printf("Rlascl test failed...\n");
	exit(-1);
    }
}

void
Rlascl_test(void)
{
    Rlascl_test2("G");
    Rlascl_test2("L");
    Rlascl_test2("U");
    Rlascl_test2("H");
    Rlascl_test2("B");
    Rlascl_test2("Q");
    Rlascl_test2("Z");
}

int
main(int argc, char *argv[])
{
//initialization
    int default_prec = 200;
    gmp_randstate_t state;

    mpf_set_default_prec(default_prec);
    uniformrandom = new gmp_randclass(gmp_randinit_default);

    Rlascl_test();
    printf("Rlascl test passed...\n");

}
