/*************************************************************************
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 * Copyright 2008 by Nakata, Maho
 *
 * MPACK - multiple precision arithmetic library
 *
 * This file is part of MPACK.
 *
 * MPACK is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 3
 * only, as published by the Free Software Foundation.
 *
 * MPACK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3 for more details
 * (a copy is included in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3 along with MPACK.  If not, see
 * <http://www.gnu.org/licenses/lgpl.html>
 * for a copy of the LGPLv3 License.
 *
 ************************************************************************/

#include <cstdio>
#include <cstdlib>
#include <mblas.h>
#include <mlapack.h>

#include "blas.h"
#include "lapack.h"
#include "mblas.debug.h"

#define MIN_N     0
#define MAX_N     10
#define MIN_M     0
#define MAX_M     10
#define MAX_LDA   15
#define MAX_ITER  10

void Rlanst_test2(const char *uplo);

void
Rlaset_test2(const char *uplo)
{
    int errorflag = FALSE;
    int j = 0;
    mpf_class Rlaset_ret;
    double dlanst_ret;

    for (int n = MIN_N; n < MAX_N; n++) {
	for (int m = MIN_M; m < MAX_M; m++) {
	    for (int lda = max(1, m); lda < MAX_LDA; lda++) {
		mpf_class *A = new mpf_class[MAT_A(lda, n)];
		double *Ad = new double[MAT_A(lda, n)];
		mpf_class alpha, beta;
		double alphad, betad;

		j = 0;
		while (j < MAX_ITER) {
		    set_random_vector(A, Ad, MAT_A(lda, n));
		    set_random_number(&alpha, &alphad);
		    set_random_number(&beta, &betad);

		    Rlaset(uplo, m, n, alpha, beta, A, lda);
		    dlaset_f77(uplo, &m, &n, &alphad, &betad, Ad, &lda);

		    double diff = vec_diff(A, Ad, MAT_A(lda, n), 1);

		    if (fabs(diff) > EPSILON) {
			printf("#n:%d m:%d lda: %d uplo %s\n", n, m, lda,
			    uplo);
			printf("error %lf!!\n", diff);
			errorflag = TRUE;
		    }
		    j++;
		}
		delete[]A;
		delete[]Ad;
	    }
	}
    }

    if (errorflag == TRUE) {
	printf("Rlaset test failed...\n");
	exit(-1);
    }
}

void
Rlaset_test(void)
{
    Rlaset_test2("U");
    Rlaset_test2("L");
}

int
main(int argc, char *argv[])
{
//initialization
    int default_prec = 200;
    gmp_randstate_t state;

    mpf_set_default_prec(default_prec);
    uniformrandom = new gmp_randclass(gmp_randinit_default);

    Rlaset_test();
    printf("Rlaset test passed...\n");

}
