/*************************************************************************
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 * Copyright 2008 by Nakata, Maho
 *
 * MPACK - multiple precision arithmetic library
 *
 * This file is part of MPACK.
 *
 * MPACK is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 3
 * only, as published by the Free Software Foundation.
 *
 * MPACK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3 for more details
 * (a copy is included in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3 along with MPACK.  If not, see
 * <http://www.gnu.org/licenses/lgpl.html>
 * for a copy of the LGPLv3 License.
 *
 ************************************************************************/

#include <cstdio>
#include <cstdlib>
#include <mblas.h>
#include <mlapack.h>

#include "blas.h"
#include "lapack.h"
#include "mblas.debug.h"

#define MIN_N     0
#define MAX_N     15
#define MIN_M     0
#define MAX_M     15
#define MAX_LDA   18
#define MAX_ITER  3

void Rlasr_test2(const char *side, const char *pivot, const char *direct);

void
Rlasr_test2(const char *side, const char *pivot, const char *direct)
{
    int errorflag = FALSE;
    int j = 0;
    int lda, m, n, cdim;

    for (int n = MIN_N; n < MAX_N; n++) {
	for (int m = MIN_M; m < MAX_M; m++) {
	    for (int lda = max(m, 1); lda < MAX_LDA; lda++) {

		printf
		    ("# n:%d m:%d lda:%d, side %s, pivot %s direct %s\n",
		    n, m, lda, side, pivot, direct);

		if (Mlsame(side, "L"))
		    cdim = m - 1;
		if (Mlsame(side, "R"))
		    cdim = n - 1;

		mpf_class *A = new mpf_class[MAT_A(lda, n)];
		mpf_class *c = new mpf_class[max(cdim, 1)];
		mpf_class *s = new mpf_class[max(cdim, 1)];
		double *Ad = new double[MAT_A(lda, n)];
		double *cd = new double[max(cdim, 1)];
		double *sd = new double[max(cdim, 1)];

		j = 0;
		while (j < MAX_ITER) {
		    set_random_vector(A, Ad, MAT_A(lda, n));
		    set_random_vector(c, cd, max(cdim, 1));
		    set_random_vector(s, sd, max(cdim, 1));
		    Rlasr(side, pivot, direct, m, n, c, s, A, lda);
		    dlasr_f77(side, pivot, direct, &m, &n, cd, sd, Ad, &lda);
		    double diff = vec_diff(A, Ad, MAT_A(lda, n), 1);

		    if (fabs(diff) > EPSILON) {
			printf
			    ("# n:%d m:%d lda:%d, side %s, pivot %s direct %s\n",
			    n, m, lda, side, pivot, direct);
			printf("error %e!!\n", diff);
			errorflag = TRUE;
		    }
		    j++;
		}
		delete[]sd;
		delete[]cd;
		delete[]Ad;
		delete[]s;
		delete[]c;
		delete[]A;
	    }
	}
    }

    if (errorflag == TRUE) {
	printf("Rlasr test failed...\n");
	exit(-1);
    }

}
void
Rlasr_test(void)
{
    Rlasr_test2("L", "V", "F");
    Rlasr_test2("R", "V", "F");
    Rlasr_test2("L", "T", "F");
    Rlasr_test2("R", "T", "F");
    Rlasr_test2("L", "B", "F");
    Rlasr_test2("R", "B", "F");
    Rlasr_test2("L", "V", "B");
    Rlasr_test2("R", "V", "B");
    Rlasr_test2("L", "T", "B");
    Rlasr_test2("R", "T", "B");
    Rlasr_test2("L", "B", "B");
    Rlasr_test2("R", "B", "B");

}

int
main(int argc, char *argv[])
{
//initialization
    int default_prec = 200;
    gmp_randstate_t state;

    mpf_set_default_prec(default_prec);
    uniformrandom = new gmp_randclass(gmp_randinit_default);

    Rlasr_test();
    printf("Rlasr test passed...\n");

}
