/*************************************************************************
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 * Copyright 2008 by Nakata, Maho
 *
 * MPACK - multiple precision arithmetic library
 *
 * This file is part of MPACK.
 *
 * MPACK is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 3
 * only, as published by the Free Software Foundation.
 *
 * MPACK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3 for more details
 * (a copy is included in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3 along with MPACK.  If not, see
 * <http://www.gnu.org/licenses/lgpl.html>
 * for a copy of the LGPLv3 License.
 *
 ************************************************************************/

#include <cstdio>
#include <cstdlib>
#include <mblas.h>
#include <mlapack.h>

#include "blas.h"
#include "lapack.h"
#include "mblas.debug.h"

#define MIN_N     0
#define MAX_N     50
#define MAX_LDA   60
#define MAX_ITER  10

void Rlasrt_test2(const char *id);

void
Rlasrt_test2(const char *id)
{
    int errorflag = FALSE;
    int j;
    int info, infod;

    for (int n = MIN_N; n < MAX_N; n++) {
	printf("#n:%d id %s\n", n, id);
	mpf_class *d = new mpf_class[VEC_A(n, 1)];
	double *dd = new double[VEC_A(n, 1)];

	j = 0;
	while (j < MAX_ITER) {
	    set_random_vector(d, dd, VEC_A(n, 1));
	    Rlasrt(id, n, d, &info);
	    dlasrt_f77(id, &n, dd, &infod);

	    if (info != infod) {
		printf("#error info info:%d infod:%d\n", info, infod);
		errorflag = TRUE;
	    }

	    double diff;

	    diff = vec_diff(d, dd, VEC_A(n, 1), 1);
	    if (fabs(diff) > EPSILON) {
		printf("#error %.16lf!!\n", diff);
		errorflag = TRUE;
	    } else {
		//printf("#okay\n");
	    }
	    j++;
	}
	delete[]d;
	delete[]dd;
    }
    if (errorflag == TRUE) {
	printf("Rlasrt test failed...\n");
	exit(-1);
    }
}


void
Rlasrt_test(void)
{
    Rlasrt_test2("I");
    Rlasrt_test2("D");

}

int
main(int argc, char *argv[])
{
//initialization
    int default_prec = 200;
    gmp_randstate_t state;

    mpf_set_default_prec(default_prec);
    uniformrandom = new gmp_randclass(gmp_randinit_default);

    Rlasrt_test();
    printf("Rlasrt test passed...\n");

}
