/*************************************************************************
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 * Copyright 2008 by Nakata, Maho
 *
 * MPACK - multiple precision arithmetic library
 *
 * This file is part of MPACK.
 *
 * MPACK is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 3
 * only, as published by the Free Software Foundation.
 *
 * MPACK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3 for more details
 * (a copy is included in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3 along with MPACK.  If not, see
 * <http://www.gnu.org/licenses/lgpl.html>
 * for a copy of the LGPLv3 License.
 *
 ************************************************************************/

#include <cstdio>
#include <cstdlib>
#include <mblas.h>
#include <mlapack.h>

#include "blas.h"
#include "lapack.h"
#include "mblas.debug.h"

#define MIN_INCX  1
#define MAX_INCX  10
#define MIN_N     0
#define MAX_N     100
#define MAX_ITER 10

void
Rlassq_test()
{
    int errorflag = FALSE;
    mpf_class scale, sumsq;
    double scaled, sumsqd;

    for (int incx = MIN_INCX; incx <= MAX_INCX; incx++) {
	for (int n = MIN_N; n <= MAX_N; n++) {
	    mpf_class *x = new mpf_class[VEC_A(n, incx)];
	    double *xd = new double[VEC_A(n, incx)];
	    int j = 0;

	    while (j < MAX_ITER) {
		set_random_vector(x, xd, VEC_A(n, incx));

		set_random_number(&scale, &scaled);
		set_random_number(&sumsq, &sumsqd);

		Rlassq(n, x, incx, &scale, &sumsq);
		dlassq_f77(&n, xd, &incx, &scaled, &sumsqd);

		double diff1 = num_diff(scale, scaled);
		double diff2 = num_diff(sumsq, sumsqd);

		if (fabs(diff1) > EPSILON) {
		    printf("error %lf!!\n", diff1);
		    errorflag = TRUE;
		}

		if (fabs(diff2) > EPSILON) {
		    printf("error %lf!!\n", diff2);
		    errorflag = TRUE;
		}

		j++;
	    }
	    delete[]x;
	    delete[]xd;
	}
    }

    if (errorflag == TRUE) {
	printf("Rlassq test failed...\n");
	exit(-1);
    }
}

int
main(int argc, char *argv[])
{
//initialization
    int default_prec = 200;
    gmp_randstate_t state;

    mpf_set_default_prec(default_prec);
    uniformrandom = new gmp_randclass(gmp_randinit_default);

    Rlassq_test();
    printf("Rlassq test passed...\n");

}
