/*************************************************************************
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 * Copyright 2008 by Nakata, Maho
 *
 * MPACK - multiple precision arithmetic library
 *
 * This file is part of MPACK.
 *
 * MPACK is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 3
 * only, as published by the Free Software Foundation.
 *
 * MPACK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3 for more details
 * (a copy is included in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3 along with MPACK.  If not, see
 * <http://www.gnu.org/licenses/lgpl.html>
 * for a copy of the LGPLv3 License.
 *
 ************************************************************************/

#include <cstdio>
#include <cstdlib>
#include <mblas.h>
#include <mlapack.h>

#include "blas.h"
#include "lapack.h"
#include "mblas.debug.h"

#define MIN_N      0
#define MAX_N     40		//should not be so large
#define MAX_LDA   45		//should not be so large
#define MAX_ITER  10

void Rpotf2_test2(const char *uplo);

void
Rpotf2_test2(const char *uplo)
{
    int errorflag = FALSE;
    int j = 0;
    int info, infod;

    for (int n = MIN_N; n < MAX_N; n++) {
	for (int lda = max(n, 1); lda < MAX_LDA; lda++) {
	    mpf_class *A = new mpf_class[MAT_A(lda, n)];
	    double *Ad = new double[MAT_A(lda, n)];

	    printf("n:%d lda %d, uplo %s\n", n, lda, uplo);
	    j = 0;
	    while (j < MAX_ITER) {
		set_random_vector(A, Ad, MAT_A(lda, n));
		Rpotf2(uplo, n, A, lda, &info);
		dpotf2_f77(uplo, &n, Ad, &lda, &infod);
		if (info != infod) {
		    printf("info error! %d, %d\n", info, infod);
		    errorflag = TRUE;
		}
		double diff = vec_diff(A, Ad, MAT_A(lda, n), 1);

		if (fabs(diff) > EPSILON3) {
		    printf("n:%d lda %d, uplo %s\n", n, lda, uplo);
		    printf("error %e!!\n", diff);
		    errorflag = TRUE;
		}
		j++;
	    }
	    delete[]A;
	    delete[]Ad;
	}
	if (errorflag == TRUE) {
	    printf("Rpotf2 test failed...\n");
	    exit(-1);
	}
    }
}

void
Rpotf2_test(void)
{
    Rpotf2_test2("L");
    Rpotf2_test2("U");
}

int
main(int argc, char *argv[])
{
//initialization
    int default_prec = 200;
    gmp_randstate_t state;

    mpf_set_default_prec(default_prec);
    uniformrandom = new gmp_randclass(gmp_randinit_default);

    Rpotf2_test();
    printf("Rpotf2 test passed...\n");

}
